/***************************************************************************
 begin       : Thu Apr 08 2010
 copyright   : (C) 2010 by Martin Preuss
 email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "globals.h"

#include <aqfinance/engine/modules/ae_statementexport.h>

#include <aqbanking/banking.h>
#include <aqbanking/types/imexporter_context.h>

#include <gwenhywfar/text.h>
#include <gwenhywfar/syncio_file.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>



int exportStm(AE_BOOK_TABLE_TYPE tt,
              AB_BANKING *ab,
              GWEN_DB_NODE *dbArgs,
              int argc,
              char **argv)
{
  GWEN_DB_NODE *db;
  int rv;
  const char *dbUrl;
  AE_BOOK *b;
  const char *outFile;
  const char *exporterName;
  const char *profileName;
  const char *profileFile;
  const char *queryString;
  AB_IMEXPORTER_CONTEXT *ctx=0;
  const GWEN_ARGS args[]= {
    {
      GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
      GWEN_ArgsType_Char,           /* type */
      "dburl",                      /* name */
      0,                            /* minnum */
      1,                            /* maxnum */
      "D",                          /* short option */
      "dburl",                      /* long option */
      I18S("Specify the database URL"),
      I18S("Specify the database URL")
    },
    {
      GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
      GWEN_ArgsType_Char,            /* type */
      "queryString",               /* name */
      0,                            /* minnum */
      1,                            /* maxnum */
      "q",                            /* short option */
      "query",                    /* long option */
      "Specify the transactions to export",   /* short description */
      "Specify the transactions to export"      /* long description */
    },
    {
      GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
      GWEN_ArgsType_Char,            /* type */
      "outFile",                    /* name */
      0,                            /* minnum */
      1,                            /* maxnum */
      "o",                          /* short option */
      "outfile",                    /* long option */
      "Specify the file to store the data in",   /* short description */
      "Specify the file to store the data in"      /* long description */
    },
    {
      GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
      GWEN_ArgsType_Char,            /* type */
      "exporterName",               /* name */
      0,                            /* minnum */
      1,                            /* maxnum */
      0,                            /* short option */
      "exporter",                    /* long option */
      "Specify the exporter to use",   /* short description */
      "Specify the exporter to use"      /* long description */
    },
    {
      GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
      GWEN_ArgsType_Char,            /* type */
      "profileName",                /* name */
      0,                            /* minnum */
      1,                            /* maxnum */
      0,                            /* short option */
      "profile",                    /* long option */
      "Specify the export profile to use",   /* short description */
      "Specify the export profile to use"      /* long description */
    },
    {
      GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
      GWEN_ArgsType_Char,            /* type */
      "profileFile",                /* name */
      0,                            /* minnum */
      1,                            /* maxnum */
      0,                            /* short option */
      "profile-file",               /* long option */
      "Specify the file to load the export profile from",/* short description */
      "Specify the file to load the export profile from" /* long description */
    },
    {
      GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST, /* flags */
      GWEN_ArgsType_Int,             /* type */
      "help",                       /* name */
      0,                            /* minnum */
      0,                            /* maxnum */
      "h",                          /* short option */
      "help",                       /* long option */
      "Show this help screen",      /* short description */
      "Show this help screen"       /* long description */
    }
  };

  db=GWEN_DB_GetGroup(dbArgs, GWEN_DB_FLAGS_DEFAULT, "local");
  rv=GWEN_Args_Check(argc, argv, 1,
                     0 /*GWEN_ARGS_MODE_ALLOW_FREEPARAM*/,
                     args,
                     db);
  if (rv==GWEN_ARGS_RESULT_ERROR) {
    fprintf(stderr, "ERROR: Could not parse arguments\n");
    return 1;
  }
  else if (rv==GWEN_ARGS_RESULT_HELP) {
    GWEN_BUFFER *ubuf;

    ubuf=GWEN_Buffer_new(0, 1024, 0, 1);
    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutType_Txt)) {
      fprintf(stderr, "ERROR: Could not create help string\n");
      return 1;
    }
    fprintf(stderr, "%s\n", GWEN_Buffer_GetStart(ubuf));
    GWEN_Buffer_free(ubuf);
    return 0;
  }

  exporterName=GWEN_DB_GetCharValue(db, "exporterName", 0, "csv");
  profileName=GWEN_DB_GetCharValue(db, "profileName", 0, "default");
  profileFile=GWEN_DB_GetCharValue(db, "profileFile", 0, 0);
  queryString=GWEN_DB_GetCharValue(db, "queryString", 0, 0);

  dbUrl=GWEN_DB_GetCharValue(db, "dbUrl", 0, NULL);

  /* init aqbanking */
  rv=AB_Banking_Init(ab);
  if (rv) {
    DBG_ERROR(0, "Error on init (%d)", rv);
    return 2;
  }

  rv=AFM_open_book(ab, dbUrl, &b);
  if (rv<0) {
    DBG_ERROR(0, "Error opening book (%d)", rv);
    return 4;
  }

  ctx=AB_ImExporterContext_new();
  rv=AE_StatementExport_ExportContext(b, ctx, tt, queryString);
  if (rv<0) {
    fprintf(stderr, "ERROR: Unable to export context (%d)\n", rv);
    AE_Book_Close(b, AQDB_ACTION_FLAGS_ABORT);
    AE_Book_free(b);
    return rv;
  }

  /* export new context */
  outFile=GWEN_DB_GetCharValue(db, "outFile", 0, 0);

  rv=AB_Banking_ExportToFileLoadProfile(ab, exporterName, ctx,  outFile, profileName, profileFile);
  if (rv<0) {
    DBG_ERROR(0, "Error exporting data (%d)", rv);
    AB_ImExporterContext_free(ctx);
    AE_Book_Close(b, AQDB_ACTION_FLAGS_ABORT);
    AE_Book_free(b);
    return 4;
  }

  AB_ImExporterContext_free(ctx);

  rv=AE_Book_Close(b, 0);
  if (rv<0) {
    DBG_ERROR(0, "Error closing book (%d)", rv);
    return 4;
  }

  rv=AB_Banking_Fini(ab);
  if (rv) {
    fprintf(stderr, "ERROR: Error on deinit (%d)\n", rv);
    return 5;
  }

  return 0;
}




