/***************************************************************************
 begin       : Sat Apr 03 2010
 copyright   : (C) 2019 by Martin Preuss
 email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/


#include <aqfinance/engine/ae.h>
#include <aqfinance/engine/book/ae_book.h>

#include <aqdatabase/aqdb_db.h>


static int get_db_url(AB_BANKING *ab, GWEN_BUFFER *pbuf)
{
  int rv;

  GWEN_Buffer_AppendString(pbuf, "dir://");
  rv=AB_Banking_GetSharedDataDir(ab, "aqfinance", pbuf);
  if (rv<0) {
    fprintf(stderr, "ERROR: Unable to get shared data dir (%d)\n", rv);
    return rv;
  }

  return 0;
}





int AFM_open_book_ro(AB_BANKING *ab, const char *url, AE_BOOK **pBook)
{
  GWEN_BUFFER *pbuf;
  AQDB_DB *db=NULL;
  AE_BOOK *b;
  int rv;
  int i;
  const AE_BOOK_TABLE_TYPE *tableList;

  rv=AE_Init(NULL);
  if (rv) {
    DBG_ERROR_ERR(0, rv);
    return rv;
  }

  pbuf=GWEN_Buffer_new(0, 256, 0, 1);
  if (url && *url)
    GWEN_Buffer_AppendString(pbuf, url);
  else {
    rv=get_db_url(ab, pbuf);
    if (rv<0) {
      GWEN_Buffer_free(pbuf);
      return rv;
    }
  }

  rv=AQDB_DbFactory(GWEN_Buffer_GetStart(pbuf), &db);
  if (rv<0) {
    fprintf(stderr, "ERROR: Unable to instantiate db (%d)\n", rv);
    GWEN_Buffer_free(pbuf);
    return rv;
  }
  GWEN_Buffer_free(pbuf);

  b=AE_Book_new(db);

  rv=AE_Book_Open(b, AQDB_ACTION_FLAGS_READ);
  if (rv<0) {
    fprintf(stderr, "ERROR: Unable to open book (%d)\n", rv);
    AE_Book_free(b);
    return rv;
  }


  tableList=AE_Book_GetTableIdList(b);
  assert(tableList);

  for (i=0; ; i++) {
    if (tableList[i]==AE_Book_TableType_Last)
      break;

    rv=AE_Book_OpenTable(b, tableList[i], AQDB_ACTION_FLAGS_READ);
    if (rv<0) {
      DBG_ERROR(0, "Unable to open table %d (%d), ignoring.", tableList[i], rv);
    }
  }

  *pBook=b;

  return 0;
}



int AFM_open_book(AB_BANKING *ab, const char *url, AE_BOOK **pBook)
{
  GWEN_BUFFER *pbuf;
  AQDB_DB *db=NULL;
  AE_BOOK *b;
  int rv;
  int i;
  const AE_BOOK_TABLE_TYPE *tableList;

  rv=AE_Init(NULL);
  if (rv) {
    DBG_ERROR_ERR(0, rv);
    return rv;
  }

  pbuf=GWEN_Buffer_new(0, 256, 0, 1);
  if (url && *url)
    GWEN_Buffer_AppendString(pbuf, url);
  else {
    rv=get_db_url(ab, pbuf);
    if (rv<0) {
      GWEN_Buffer_free(pbuf);
      return rv;
    }
  }

  rv=AQDB_DbFactory(GWEN_Buffer_GetStart(pbuf), &db);
  if (rv<0) {
    fprintf(stderr, "ERROR: Unable to instantiate db (%d)\n", rv);
    GWEN_Buffer_free(pbuf);
    return rv;
  }
  GWEN_Buffer_free(pbuf);

  b=AE_Book_new(db);

  rv=AE_Book_Open(b, AQDB_ACTION_FLAGS_READ | AQDB_ACTION_FLAGS_WRITE);
  if (rv<0) {
    fprintf(stderr, "ERROR: Unable to open book (%d)\n", rv);
    AE_Book_free(b);
    return rv;
  }


  tableList=AE_Book_GetTableIdList(b);
  assert(tableList);

  for (i=0; ; i++) {
    if (tableList[i]==AE_Book_TableType_Last)
      break;

    rv=AE_Book_OpenTable(b, tableList[i], AQDB_ACTION_FLAGS_READ | AQDB_ACTION_FLAGS_WRITE);
    if (rv<0) {
      int rv2;

      DBG_ERROR(0, "Table %d does not exist, creating", tableList[i]);
      rv2=AE_Book_CreateTable(b, tableList[i], AQDB_ACTION_FLAGS_READ | AQDB_ACTION_FLAGS_WRITE);
      if (rv2==0) {
        DBG_ERROR(0, "  Retrying to open table %d after creating it", tableList[i]);
        rv=AE_Book_OpenTable(b, tableList[i], AQDB_ACTION_FLAGS_READ | AQDB_ACTION_FLAGS_WRITE);
        if (rv<0) {
          DBG_ERROR(0, "Unable to open table type %d (%d)", tableList[i], rv);
          AE_Book_free(b);
          return rv;
        }
        else {
          DBG_ERROR(0, "  Ok, table %d is now open", tableList[i]);
        }
      }
      else {
        DBG_ERROR(0, "Unable to create table type %d (%d)", tableList[i], rv);
        AE_Book_free(b);
        return rv;
      }
    }
  }

  *pBook=b;

  return 0;
}



int AFM_create_book(AB_BANKING *ab,
                    const char *url)
{
  GWEN_BUFFER *pbuf;
  AQDB_DB *db=NULL;
  AE_BOOK *b;
  int rv;
  int i;
  const AE_BOOK_TABLE_TYPE *tableList;

  rv=AE_Init(NULL);
  if (rv) {
    DBG_ERROR_ERR(0, rv);
    return rv;
  }

  pbuf=GWEN_Buffer_new(0, 256, 0, 1);
  if (url && *url)
    GWEN_Buffer_AppendString(pbuf, url);
  else {
    rv=get_db_url(ab, pbuf);
    if (rv<0) {
      GWEN_Buffer_free(pbuf);
      return rv;
    }
  }

  rv=AQDB_DbFactory(GWEN_Buffer_GetStart(pbuf), &db);
  if (rv<0) {
    fprintf(stderr, "ERROR: Unable to instantiate db (%d)\n", rv);
    GWEN_Buffer_free(pbuf);
    return rv;
  }
  GWEN_Buffer_free(pbuf);

  b=AE_Book_new(db);

  rv=AE_Book_Create(b, 0);
  if (rv<0) {
    fprintf(stderr, "ERROR: Unable to create book (%d)\n", rv);
    AE_Book_free(b);
    return rv;
  }

  rv=AE_Book_Open(b, AQDB_ACTION_FLAGS_READ | AQDB_ACTION_FLAGS_WRITE);
  if (rv<0) {
    fprintf(stderr, "ERROR: Unable to open book (%d)\n", rv);
    AE_Book_free(b);
    return rv;
  }


  tableList=AE_Book_GetTableIdList(b);
  assert(tableList);

  for (i=0; ; i++) {
    if (tableList[i]==AE_Book_TableType_Last)
      break;

    rv=AE_Book_CreateTable(b, tableList[i], AQDB_ACTION_FLAGS_READ | AQDB_ACTION_FLAGS_WRITE);
    if (rv<0) {
      DBG_ERROR(0, "Unable to open table %d (%d), ignoring.", tableList[i], rv);
    }
  }

  AE_Book_free(b);

  return 0;
}



