/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2018 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_editstmrule.hpp"

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)
#define I18S(msg) msg


static FF_EditRule::FieldDescription _fields[]={
  {"id",                  I18S("Id")},
  {"bankAccountId",       I18S("Bank Account Id")},
  {"localCountry",        I18S("Local Country")},
  {"localBankCode",       I18S("Local Bank Code")},
  {"localAccountNumber",  I18S("Local Account Number")},
  {"localiban",           I18S("Local IBAN")},
  {"localbic",            I18S("Local BIC")},
  {"localname",           I18S("Local Name")},
  {"remoteCountry",       I18S("Remote Country")},
  {"remoteBankCode",      I18S("Remote Bank Code")},
  {"remoteAccountNumber", I18S("Remote Account Number")},
  {"remoteiban",          I18S("Remote IBAN")},
  {"remotebic",           I18S("Remote BIC")},
  {"remotename",          I18S("Remote Name")},
  {"date",                I18S("Date")},
  {"valutadate",          I18S("Valuta Date")},
  {"amount",              I18S("Value (Amount)")},
  {"currency",            I18S("Value (Currency)")},
  {"fiid",                I18S("Id from Financial Institute")},
  {"textkey",             I18S("Text Key")},
  {"textkeyext",          I18S("Text Key Extension")},
  {"infoText",            I18S("Info Text")},
  {"customerRef",         I18S("Customer Reference")},
  {"bankRef",             I18S("Bank Reference")},
  {"purpose",             I18S("Purpose")},
  {"status",              I18S("Status")},
  {"transferid",          I18S("Transfer Id")},
  {"groupid",             I18S("Group Id")},
  {"category",            I18S("Category")},
  {"endToEndId",          I18S("endToEndId")},
  {NULL, NULL}
};



FF_EditStatementRule::FF_EditStatementRule(FF_App* a,
                                           FXComposite *parent,
                                           const FXString& title,
                                           FXuint opts,
                                           FXint x, FXint y, FXint w, FXint h,
                                           FXint pl, FXint pr, FXint pt, FXint pb,
                                           FXint hs, FXint vs)
:FF_EditRule(a, _fields, parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_dbExample(NULL)
{
}



FF_EditStatementRule::FF_EditStatementRule()
:FF_EditRule()
,m_dbExample(NULL)
{
}



FF_EditStatementRule::~FF_EditStatementRule() {
  if (m_dbExample) {
    GWEN_DB_Group_free(m_dbExample);
    m_dbExample=NULL;
  }
}



FXint FF_EditStatementRule::toGui(const AE_RULE *r, GWEN_DB_NODE *dbExample) {
  if (m_dbExample) {
    GWEN_DB_Group_free(m_dbExample);
    m_dbExample=NULL;
  }
  if (dbExample) {
    m_dbExample=GWEN_DB_Group_dup(dbExample);
  }

  return FF_EditRule::toGui(r);
}



FXString FF_EditStatementRule::getExampleValue(FXint entryNum) {
  if (m_dbExample) {
    FXString str=_fields[entryNum].name;

    if (!str.empty()) {
      /* handle special fields first */
      if (comparecase(str, "amount")==0) {
        const char *cfv;

        cfv=GWEN_DB_GetCharValue(m_dbExample, "value", 0, NULL);
        if (cfv && *cfv) {
          AQDB_VALUE *v;

          v=AQDB_Value_fromString(cfv);
          if (v) {
            GWEN_BUFFER *tbuf;
            FXString res;

            tbuf=GWEN_Buffer_new(0, 256, 0, 1);
            AQDB_Value_toHumanReadableString(v, tbuf, 2);
            res=FXString(GWEN_Buffer_GetStart(tbuf));
            GWEN_Buffer_free(tbuf);
            AQDB_Value_free(v);
            return res;
          }
        }
      }

      switch(GWEN_DB_GetVariableType(m_dbExample, str.text())) {
      case GWEN_DB_NodeType_ValueChar:
        {
          const char *cfv;

          cfv=GWEN_DB_GetCharValue(m_dbExample, str.text(), 0, NULL);
          if (cfv && *cfv)
            return FXString(cfv);
          break;
        }

      case GWEN_DB_NodeType_ValueInt:
        {
          int cfv;

          cfv=GWEN_DB_GetIntValue(m_dbExample, str.text(), 0, 0);
          return FXStringVal(cfv, 10);
          break;
        }
      default:
        break;
      } /* switch */
    } /* if (!str.empty()) */
  } /* if (m_dbExample) */

  return "";
}



bool FF_EditStatementRule::editRule(FF_App* app,
                                    FXComposite *parent,
                                    const FXString &title,
                                    AE_RULE *r,
                                    GWEN_DB_NODE *dbExample) {
  FF_EditStatementRule dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;

  dlg.addDialogLine();
  dlg.resize(440, 400);
  dlg.create();
  dlg.init();
  dlg.toGui(r, dbExample);

  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  if (dlgres==0)
    return false;

  dlg.fromGui(r);
  return true;
}



