/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_selectabaccs.hpp"
#include "ff_app.hpp"
#include "ff_bankaccounttree.hpp"

#include <aqfinance/engine/book/ae_bankaccount.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_SelectAbAccounts) FF_SelectAbAccountMap[]={
  FXMAPFUNC(SEL_LEFTBUTTONPRESS, FF_SelectAbAccounts::ID_ACCOUNT_LIST, FF_SelectAbAccounts::onLeftButtonPress),

  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_SelectAbAccounts::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_SelectAbAccounts::onCmdCancel),

};


FXIMPLEMENT(FF_SelectAbAccounts, FXDialogBox,
            FF_SelectAbAccountMap, ARRAYNUMBER(FF_SelectAbAccountMap))




FF_SelectAbAccounts::FF_SelectAbAccounts(FF_App* a,
                                         FXComposite *parent,
                                         const FXString& title,
                                         const FXString& text,
                                         FXuint opts,
                                         FXint x, FXint y, FXint w, FXint h,
                                         FXint pl, FXint pr, FXint pt, FXint pb,
                                         FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_label(NULL)
,m_accountTree(NULL)
{
  FXVerticalFrame *vf;
  FXHorizontalFrame *hf;

  vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_label=new FOX16_HtmlLabel(vf, text, LAYOUT_FILL_X);
  m_label->setMinimumWidth(150);
  m_accountTree=new FF_BankAccountTree(vf, this, ID_ACCOUNT_LIST,
                                       LAYOUT_FILL_X | LAYOUT_FILL_Y |
                                       LIST_SINGLESELECT |
                                       FXXCheckTree::OPTIONS_CHECKMARKS);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
}



FF_SelectAbAccounts::FF_SelectAbAccounts()
:FXDialogBox()
,m_app(NULL)
,m_label(NULL)
,m_accountTree(NULL)
{
}



FF_SelectAbAccounts::~FF_SelectAbAccounts() {
}



void FF_SelectAbAccounts::init() {
  GWEN_DB_NODE *db;
  int w, h;
  int i;
  int sortByColumn;
  int sortUp;

  db=m_app->getDialogConfig("FF_SelectAbAccounts");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight()+100);

  sortByColumn=GWEN_DB_GetIntValue(db, "sortByColumn", 0, -1);
  sortUp=GWEN_DB_GetIntValue(db, "sortUp", 0, 0);

  for (i=0; i<m_accountTree->getNumHeaders(); i++) {
    int j=GWEN_DB_GetIntValue(db, "columns", i, m_accountTree->getHeaderSize(i));
    m_accountTree->setHeaderSize(i, j);
    if (i==sortByColumn) {
      if (sortUp)
        m_accountTree->setHeaderArrowDir(i, TRUE);
      else
        m_accountTree->setHeaderArrowDir(i, FALSE);
    }
  }

  GWEN_DB_Group_free(db);
}



void FF_SelectAbAccounts::fini() {
  GWEN_DB_NODE *db;
  int rv;
  int i;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  for (i=0; i<m_accountTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT, "columns",
                        m_accountTree->getHeaderSize(i));
    b=m_accountTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
        GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
        GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }

  rv=m_app->setDialogConfig("FF_SelectAbAccounts", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}




FXint FF_SelectAbAccounts::checkGui(FXbool quiet) {
#if 0
  if (m_accountTree->getFirstSelectedBankAccount())
    return 0;
  return GWEN_ERROR_GENERIC;
#else
  return 0;
#endif
}



void FF_SelectAbAccounts::toGui(const AB_ACCOUNT_SPEC_LIST *al) {
  AB_ACCOUNT_SPEC *as;

  as=AB_AccountSpec_List_First(al);
  while(as) {
    AE_BANKACCOUNT *a=AE_BankAccount_new();
    const char *s;

    s=AB_AccountSpec_GetAccountName(as);
    AE_BankAccount_SetAccountName(a, s);

    s=AB_AccountSpec_GetAccountNumber(as);
    AE_BankAccount_SetAccountNumber(a, s);

    s=AB_AccountSpec_GetBankCode(as);
    AE_BankAccount_SetBankCode(a, s);

    s=AB_AccountSpec_GetBankName(as);
    AE_BankAccount_SetBankName(a, s);

    s=AB_AccountSpec_GetBic(as);
    AE_BankAccount_SetBic(a, s);

    s=AB_AccountSpec_GetIban(as);
    AE_BankAccount_SetIban(a, s);

    s=AB_AccountSpec_GetOwnerName(as);
    AE_BankAccount_SetOwnerName(a, s);

    AE_BankAccount_SetBankingId(a, AB_AccountSpec_GetUniqueId(as));
    m_accountTree->addAccount(a, NULL, NULL);

    as=AB_AccountSpec_List_Next(as);
  } /* while (as) */

  m_accountTree->killSelection();
}



AE_BANKACCOUNT_LIST *FF_SelectAbAccounts::fromGui() {
  AE_BANKACCOUNT_LIST *bal;
  FXFoldingItem *item;

  bal=AE_BankAccount_List_new();

  item=m_accountTree->getFirstItem();
  while (item) {
    FF_BankAccountTreeItem *baItem=dynamic_cast<FF_BankAccountTreeItem*>(item);
    if (baItem) {
      AE_BANKACCOUNT *ba;

      ba=baItem->getBankAccount();
      /* only add checked items */
      if (ba && baItem->isChecked())
        AE_BankAccount_List_Add(AE_BankAccount_dup(ba), bal);
    }

    item=item->getBelow();
  }

  if (AE_BankAccount_List_GetCount(bal)==0) {
    DBG_ERROR(0, "No account selected");
    AE_BankAccount_List_free(bal);
    return NULL;
  }

  return bal;
}



void FF_SelectAbAccounts::create() {
  FXDialogBox::create();
  m_label->create();
  m_accountTree->create();
}



long FF_SelectAbAccounts::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  getApp()->stopModal(this,TRUE);
  hide();
  return 1;
}



long FF_SelectAbAccounts::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  hide();
  return 1;
}



AE_BANKACCOUNT_LIST *FF_SelectAbAccounts::selectAccounts(FF_App* app,
                                                         FXComposite *parent,
                                                         const FXString &title,
                                                         const FXString& text,
                                                         const AB_ACCOUNT_SPEC_LIST *accountSpecList) {
  FF_SelectAbAccounts dlg(app, parent, title, text, DECOR_ALL);
  FXuint dlgres;

  dlg.resize(440, 400);
  dlg.create();
  dlg.init();
  dlg.toGui(accountSpecList);
  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    /* aborted */
    return NULL;
  }

  return dlg.fromGui();
}




long FF_SelectAbAccounts::onLeftButtonPress(FXObject*, FXSelector, void *ptr) {
  FXEvent* event=(FXEvent*)ptr;
  FXFoldingItem *item;
  FXint code;

  item=m_accountTree->getItemAt(event->win_x,event->win_y);
  if (item) {
    code=m_accountTree->hitItem(item,event->win_x,event->win_y);
    if (code==4) {
      FF_BankAccountTreeItem *aitem=dynamic_cast<FF_BankAccountTreeItem*>(item);

      if (aitem) {
	/* checkbox clicked */
	aitem->setChecked(!(aitem->isChecked()));
        m_accountTree->updateItem(aitem);
        return 1;
      }
    }
  }

  return 0;
}



