/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_gui.hpp"
#include "ff_app.hpp"
#include "ff_flickerdialog.hpp"
#include "ff_tanimagedialog.hpp"

#include <FXPNGImage.h>


#ifdef HAVE_CAIRO
# include <aqfinance/print/af_print.h>
# include <aqfinance/print/af_print_doc.h>
#endif


#include <gwenhywfar/directory.h>
#include <gwenhywfar/fslock.h>
#include <gwenhywfar/text.h>

#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>



#define LOGFILE_NAME    "log"
#define LOGFILE_BACKLOG 5



FF_Gui::FF_Gui(FF_App* a, const FXString &dataPath)
:FOX16_Gui(a)
,m_hookActive(false)
,m_maxLogLevel(GWEN_LoggerLevel_Info)
,m_dataPath(dataPath)
{
}



FF_Gui::~FF_Gui() {
}



int FF_Gui::logHook(const char *logDomain,
		    GWEN_LOGGER_LEVEL priority, const char *s) {
  if (!m_hookActive)
    return 0;
  else {
    if (priority<=m_maxLogLevel) {
      GWEN_FSLOCK *lck;
      GWEN_FSLOCK_RESULT res;
      FILE *f;
      GWEN_BUFFER *tbuf;
      int rv;
  
      tbuf=GWEN_Buffer_new(0, 256, 0, 1);
      rv=GWEN_Logger_CreateLogMessage(logDomain,
				      priority, s,
				      tbuf);
      if (rv<0) {
	m_hookActive=false;
	fprintf(stderr, "Unable to generate log message, turning off logging.\n");
	GWEN_Buffer_free(tbuf);
	return 0;
      }
  
      if (GWEN_Directory_GetPath(m_logFile.text(),
				 GWEN_PATH_FLAGS_VARIABLE)) {
	m_hookActive=false;
	fprintf(stderr, "Unable to get logfile, turning off logging.\n");
	GWEN_Buffer_free(tbuf);
	return 0;
      }
  
      lck=GWEN_FSLock_new(m_logFile.text(), GWEN_FSLock_TypeFile);
      res=GWEN_FSLock_Lock(lck, 30, 0);
      if (res!=GWEN_FSLock_ResultOk) {
	m_hookActive=false;
	fprintf(stderr, "Unable to lock logfile, turning off logging.\n");
	GWEN_FSLock_free(lck);
	GWEN_Buffer_free(tbuf);
	return 0;
      }
  
      f=fopen(m_logFile.text(), "a");
      if (f==NULL) {
	fprintf(stderr, "Unable to open logfile, turning off logging.\n");
	GWEN_FSLock_Unlock(lck);
	GWEN_FSLock_free(lck);
	GWEN_Buffer_free(tbuf);
	return 0;
      }
  
      /* write log message */
      fprintf(f, "%s", GWEN_Buffer_GetStart(tbuf));
  
      /* close log file */
      if (fclose(f)) {
	fprintf(stderr, "Unable to close logfile, turning off logging.\n");
	GWEN_FSLock_Unlock(lck);
	GWEN_FSLock_free(lck);
	GWEN_Buffer_free(tbuf);
	return 0;
      }
  
      /* unlock log file */
      GWEN_FSLock_Unlock(lck);
      GWEN_FSLock_free(lck);
      GWEN_Buffer_free(tbuf);
      /* log message handled */
      return 1;
    }
    else
      /* don't write debug message to file */
      return 0;
  }
}



void FF_Gui::setupLogging(bool withDebug) {
  FXString fname;
  int i;

  fname=m_dataPath;
  fname+=GWEN_DIR_SEPARATOR_S;
  fname+=LOGFILE_NAME;
  m_logFile=fname;

  if (withDebug)
    m_maxLogLevel=GWEN_LoggerLevel_Verbous;
  else
    m_maxLogLevel=GWEN_LoggerLevel_Info;

  /* cycle logfiles */
  for (i=LOGFILE_BACKLOG; i>0; i--) {
    FXString f1;
    FXString f2;

    if (i==1)
      /* i-1 is 0, so don't add number */
      f1=fname;
    else
      f1=fname+"."+FXStringVal(i-1)+".bak";
    f2=fname+"."+FXStringVal(i)+".bak";

    if (0==GWEN_Directory_GetPath(f2.text(),
				  GWEN_PATH_FLAGS_PATHMUSTEXIST |
				  GWEN_PATH_FLAGS_VARIABLE)) {
      /* remove last file */
      unlink(f2.text());
    }

    /* move file */
    if (0==GWEN_Directory_GetPath(f1.text(),
				  GWEN_PATH_FLAGS_PATHMUSTEXIST |
				  GWEN_PATH_FLAGS_VARIABLE)) {
      if (rename(f1.text(), f2.text())) {
	DBG_ERROR(0, "Unable to rename [%s] to [%s]: %s",
		  f1.text(), f2.text(),
		  strerror(errno));
        return;
      }
    }
  }

  m_hookActive=true;
}



int FF_Gui::getPasswordText(uint32_t flags,
                            const char *token,
                            const char *title,
                            const char *text,
                            char *buffer,
                            int minLen,
                            int maxLen,
                            GWEN_GUI_PASSWORD_METHOD methodId,
                            GWEN_DB_NODE *methodParams,
                            uint32_t guiid) {
  DBG_ERROR(AE_LOGDOMAIN, "GetPassword with text method, using normal dialog");
  return FOX16_Gui::getPassword(flags, token, title, text, buffer, minLen, maxLen, methodId, methodParams, guiid);
}



int FF_Gui::getPasswordHhd(uint32_t flags,
			   const char *token,
			   const char *title,
			   const char *text,
			   char *buffer,
			   int minLen,
			   int maxLen,
			   GWEN_GUI_PASSWORD_METHOD methodId,
			   GWEN_DB_NODE *methodParams,
			   uint32_t guiid) {
  int tanMethodId;

  DBG_ERROR(AE_LOGDOMAIN, "GetPassword called with HHD data");

  tanMethodId=GWEN_DB_GetIntValue(methodParams, "tanMethodId", 0, AB_BANKING_TANMETHOD_TEXT);

  if (1) {
    DBG_ERROR(AE_LOGDOMAIN, "  method is [%d]", tanMethodId);
    DBG_ERROR(AE_LOGDOMAIN, "  text is [%s]", text);
    DBG_ERROR(AE_LOGDOMAIN, "  HHD text is [%s]", text?text:"<empty>");
    DBG_ERROR(AE_LOGDOMAIN, "  title is [%s]", title?title:"<empty>");
    DBG_ERROR(AE_LOGDOMAIN, "  token is [%s]", token?token:"<empty>");
  }

  switch(tanMethodId) {
  case AB_BANKING_TANMETHOD_CHIPTAN_OPTIC:
    return getPasswordChipTanOptic(flags, token, title, text, buffer, minLen, maxLen, methodId, methodParams, guiid);

  case AB_BANKING_TANMETHOD_CHIPTAN_QR:
  case AB_BANKING_TANMETHOD_PHOTOTAN:
    return getPasswordImage(flags, token, title, text, buffer, minLen, maxLen, methodId, methodParams, guiid);

  case AB_BANKING_TANMETHOD_CHIPTAN_USB:
    break;
  default:
  case AB_BANKING_TANMETHOD_TEXT:
  case AB_BANKING_TANMETHOD_CHIPTAN:
    return getPasswordText(flags, token, title, text, buffer, minLen, maxLen, methodId, methodParams, guiid);
  }

  DBG_ERROR(AE_LOGDOMAIN, "Unhandled password method id %d", tanMethodId);
  return GWEN_ERROR_INVALID;
}



int FF_Gui::getPasswordChipTanOptic(uint32_t flags,
                                    const char *token,
                                    const char *title,
                                    const char *text,
                                    char *buffer,
                                    int minLen,
                                    int maxLen,
                                    GWEN_GUI_PASSWORD_METHOD methodId,
                                    GWEN_DB_NODE *methodParams,
                                    uint32_t guiid) {
  FXWindow *owner;
  FXString msg;
  FXString code;
  FXString result;
  const char *sChallenge;

  DBG_ERROR(AE_LOGDOMAIN, "GetPassword called with optical data");

  /* get main window of parent dialog (if any) */
  owner=m_app->getActiveWindow();

  sChallenge=GWEN_DB_GetCharValue(methodParams, "challenge", 0, NULL);
  if (!(sChallenge && *sChallenge)) {
    DBG_ERROR(AE_LOGDOMAIN, "Empty optical data");
    return GWEN_ERROR_NO_DATA;
  }
  DBG_ERROR(AE_LOGDOMAIN, "  HHD code is [%s]", sChallenge?sChallenge:"<empty>");

  /* cut out flicker code and message */
  msg=FXString(text);
  code=FXString(sChallenge);

  /* call FlickerDialog */
  if (FF_FlickerDialog::getTan(dynamic_cast<FF_App*>(m_app), owner,
			       title, msg, code, minLen, maxLen-1, result)) {
    int len;

    DBG_ERROR(AE_LOGDOMAIN, "  TAN is [%s]", result.text());

    len=result.length();
    if (len>=minLen && len<maxLen) {
      strncpy(buffer, result.text(), len);
      buffer[len]=0;
      DBG_ERROR(AE_LOGDOMAIN, "  GetTAN succeeded");
      return 0;
    }
    else {
      DBG_ERROR(AE_LOGDOMAIN, "FlickerDialog returned TAN with invalid length (%d, %d-%d)",
		len, minLen, maxLen);
      return GWEN_ERROR_INTERNAL;
    }
  }
  else {
    DBG_ERROR(AE_LOGDOMAIN, "GetTAN aborted by user");
    return GWEN_ERROR_USER_ABORTED;
  }
}



int FF_Gui::getPasswordImage(uint32_t flags,
                             const char *token,
                             const char *title,
                             const char *text,
                             char *buffer,
                             int minLen,
                             int maxLen,
                             GWEN_GUI_PASSWORD_METHOD methodId,
                             GWEN_DB_NODE *methodParams,
                             uint32_t guiid) {
  FXWindow *owner;
  FXString msg;
  FXString result;
  FXString mimeType;
  const char *s;

  DBG_ERROR(AE_LOGDOMAIN, "GetPassword called with image data");

  /* get main window of parent dialog (if any) */
  owner=m_app->getActiveWindow();

  msg=FXString(text);
 

  s=GWEN_DB_GetCharValue(methodParams, "mimeType", 0, NULL);
  if (s && *s) {
    const char *pChallenge;
    uint32_t lChallenge;
    FXImage *image=NULL;

    DBG_ERROR(AE_LOGDOMAIN, "  Mimetype is [%s]", s);

    pChallenge=(const char*) GWEN_DB_GetBinValue(methodParams, "imageData", 0, NULL, 0, &lChallenge);
    if (!(pChallenge && lChallenge)) {
      DBG_ERROR(AE_LOGDOMAIN, "Empty optical data");
      GWEN_DB_Dump(methodParams, 2);
      return GWEN_ERROR_NO_DATA;
    }

    DBG_ERROR(AE_LOGDOMAIN, "  Challenge is:");
    GWEN_Text_LogString(pChallenge, lChallenge, AE_LOGDOMAIN, GWEN_LoggerLevel_Error);

    if (strcasecmp(s, "image/png")==0) {
      image=new FXPNGImage(m_app, (const void*) pChallenge, 0);
      if (image==NULL) {
	DBG_ERROR(AE_LOGDOMAIN, "Not a PNG image.");
	return GWEN_ERROR_BAD_DATA;
      }
    }
    else {
      DBG_ERROR(AE_LOGDOMAIN, "Unhandled mime type \"%s\"", s);
      return GWEN_ERROR_NOT_SUPPORTED;
    }

    if (FF_TanImageDialog::getTan(dynamic_cast<FF_App*>(m_app), owner,
				  title, msg, image, minLen, maxLen-1, result)) {
      int len;

      DBG_ERROR(AE_LOGDOMAIN, "  TAN is [%s]", result.text());

      len=result.length();
      if (len>=minLen && len<maxLen) {
	strncpy(buffer, result.text(), len);
	buffer[len]=0;
	DBG_ERROR(AE_LOGDOMAIN, "  GetTAN succeeded");
	delete image;
	return 0;
      }
      else {
	DBG_ERROR(AE_LOGDOMAIN, "TanImageDialog returned TAN with invalid length (%d, %d-%d)",
		  len, minLen, maxLen);
	delete image;
	return GWEN_ERROR_INTERNAL;
      }
    }
    else {
      DBG_ERROR(AE_LOGDOMAIN, "GetTAN aborted by user");
      delete image;
      return GWEN_ERROR_USER_ABORTED;
    }
  }
  else {
    DBG_ERROR(AE_LOGDOMAIN, "No mimetype specified in tan params");
    return GWEN_ERROR_INVALID;
  }
}



int FF_Gui::getPassword(uint32_t flags,
                        const char *token,
                        const char *title,
                        const char *text,
                        char *buffer,
                        int minLen,
                        int maxLen,
                        GWEN_GUI_PASSWORD_METHOD methodId,
                        GWEN_DB_NODE *methodParams,
                        uint32_t guiid) {
  switch( (methodId & GWEN_Gui_PasswordMethod_Mask)) {
  case GWEN_Gui_PasswordMethod_Unknown:
  case GWEN_Gui_PasswordMethod_Mask:
    DBG_ERROR(AE_LOGDOMAIN, "Invalid password method id %08x", methodId);
    return GWEN_ERROR_INVALID;

  case GWEN_Gui_PasswordMethod_Text:
    return getPasswordText(flags,
			   token,
			   title,
			   text,
			   buffer,
			   minLen,
			   maxLen,
			   methodId, methodParams,
			   guiid);

  case GWEN_Gui_PasswordMethod_OpticalHHD:
    return getPasswordHhd(flags,
			  token,
			  title,
			  text,
			  buffer,
			  minLen,
			  maxLen,
			  methodId, methodParams,
			  guiid);

    /* intentionally omit "default:" here to be informed when new password methods are added to
     * gwen which have not been implemented. */
  }

  DBG_ERROR(AE_LOGDOMAIN, "Unhandled password method id %08x", methodId);
  return GWEN_ERROR_INVALID;
}




int FF_Gui::print(const char *docTitle,
                  const char *docType,
                  const char *descr,
                  const char *text,
                  uint32_t guiid) {
  AF_PRINT_DOC *tmpl;
  AF_PRINT *pr;
  char pbuf[300];
  GWEN_TIME *ti;
  GWEN_BUFFER *tbuf;
  FXString htmlText;
  int rv;
  GWEN_DB_NODE *db;
  const char *s;

  tmpl=AF_Print_GenerateTemplate_A4(300);
  if (tmpl==NULL) {
    DBG_ERROR(0, "No PRINT DOC object created.");
    return GWEN_ERROR_INTERNAL;
  }

  /* create file name */
  rv=GWEN_Directory_GetTmpDirectory(pbuf, sizeof(pbuf)-1);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
    AF_PrintDoc_free(tmpl);
    return GWEN_ERROR_INTERNAL;
  }
  pbuf[sizeof(pbuf)-1]=0;

  tbuf=GWEN_Buffer_new(0, 256, 0, 1);
  GWEN_Buffer_AppendString(tbuf, pbuf);
  GWEN_Buffer_AppendString(tbuf, GWEN_DIR_SEPARATOR_S);

  ti=GWEN_CurrentTime();
  assert(ti);
  GWEN_Time_toString(ti, "YYYYMMDDhhmmss", tbuf);
  GWEN_Buffer_AppendString(tbuf, ".pdf");
  GWEN_Time_free(ti);


  /* create PDF document */
  pr=AF_Print_newForPdf(GWEN_Buffer_GetStart(tbuf), tmpl);
  if (pr==NULL) {
    DBG_ERROR(0, "No PRINT object created.");
    GWEN_Buffer_free(tbuf);
    AF_PrintDoc_free(tmpl);
    return GWEN_ERROR_INTERNAL;
  }

  htmlText=getHtmlText(text);
  rv=AF_Print_CheckText(pr, AF_PrintBox_Id_ContentBox, htmlText.text());
  if (rv!=0) {
    DBG_ERROR(0, "here (%d)", rv);
    GWEN_Buffer_free(tbuf);
    AF_Print_free(pr);
    return GWEN_ERROR_INTERNAL;
  }
  AF_Print_SetText(pr, AF_PrintBox_Id_ContentBox, htmlText.text());

  rv=AF_Print_Finish(pr);
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
    GWEN_Buffer_free(tbuf);
    AF_Print_free(pr);
    return rv;
  }


  /* start PDF viewer for this file */
  db=GWEN_DB_Group_new("args");
  GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "fileName", GWEN_Buffer_GetStart(tbuf));

  s=AE_GetUserDataFolder();
  if (s && *s)
    GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "userdatadir", s);

  rv=(dynamic_cast<FF_App*>(m_app))->viewPdfFile(GWEN_Buffer_GetStart(tbuf), db);
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
    GWEN_DB_Group_free(db);
    GWEN_Buffer_free(tbuf);
    AF_Print_free(pr);
    return rv;
  }

  /* remove file */
  //unlink(GWEN_Buffer_GetStart(tbuf));

  /* done */
  GWEN_DB_Group_free(db);
  GWEN_Buffer_free(tbuf);
  AF_Print_free(pr);

  return 0;
}




