/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2014 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "fm_countersvtab.hpp"
#include "fm_counterspage.hpp"
#include "ff_countervaluetree.hpp"
#include "ff_app.hpp"
#include "ff_editgroup.hpp"
#include "ff_editcountervalue.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FM_CountersValueTab) FM_CountersValueTabMap[]={
  FXMAPFUNC(SEL_COMMAND, FM_CountersValueTab::ID_BUTTON_ADDVALUE,
            FM_CountersValueTab::onCmdAddValue),
  FXMAPFUNC(SEL_COMMAND, FM_CountersValueTab::ID_BUTTON_EDITVALUE,
            FM_CountersValueTab::onCmdEditValue),
  FXMAPFUNC(SEL_COMMAND, FM_CountersValueTab::ID_BUTTON_DELVALUE,
            FM_CountersValueTab::onCmdDelValue),

  FXMAPFUNC(SEL_CHANGED, FM_CountersValueTab::ID_VALUE_TREE,
            FM_CountersValueTab::onTreeChanged),
  FXMAPFUNC(SEL_DOUBLECLICKED, FM_CountersValueTab::ID_VALUE_TREE,
            FM_CountersValueTab::onTreeCommand),
  FXMAPFUNC(SEL_RIGHTBUTTONPRESS, FM_CountersValueTab::ID_VALUE_TREE,
            FM_CountersValueTab::onTreeRightBtnPress),

};



FXIMPLEMENT(FM_CountersValueTab, FF_ModTabPage,
            FM_CountersValueTabMap, ARRAYNUMBER(FM_CountersValueTabMap))






FM_CountersValueTab::FM_CountersValueTab(const AE_COUNTER_TYPE *vehicle,
                                         const FXString &pageId,
                                         FM_CountersPage *mainPage,
                                         FXuint opts,
                                         FXint x, FXint y, FXint w, FXint h)
:FF_ModTabPage(pageId, mainPage, opts, x, y, w, h)
,m_mainPage(mainPage)
,m_counterType(NULL)
,m_valueTree(NULL)
,m_numValues(1)
,m_isEditing(false)
{
  FXIcon *ic;
  FF_App *app;

  m_closeable=true;

  assert(vehicle);
  m_counterType=AE_CounterType_dup(vehicle);

  flags|=FLAG_ENABLED|FLAG_SHOWN;
  app=dynamic_cast<FF_App*>(getApp());

  m_valueTree=new FF_CounterValueTree(this, this, ID_VALUE_TREE,
                                       LAYOUT_SIDE_LEFT |
                                       LAYOUT_FILL_X | LAYOUT_FILL_Y |
                                       FOLDINGLIST_EXTENDEDSELECT |
                                       FOLDINGLIST_SHOWS_LINES |
                                       FOLDINGLIST_SHOWS_BOXES |
                                       FOLDINGLIST_ROOT_BOXES |
                                       FRAME_SUNKEN|FRAME_THICK);
  ic=app->getIcon("minifolderopen");
  if (ic)
    m_valueTree->setIconOpen(ic);
  ic=app->getIcon("minifolder");
  if (ic)
    m_valueTree->setIconClosed(ic);

  if (m_counterType) {
    const char *s;
    int x=0;

    s=AE_CounterType_GetXLabel(m_counterType);
    if (s && *s)
      x|=1;
    s=AE_CounterType_GetYLabel(m_counterType);
    if (s && *s)
      x|=2;
    s=AE_CounterType_GetZLabel(m_counterType);
    if (s && *s)
      x|=4;

    if (x & 4)
      m_numValues=3;
    else if (x & 2)
      m_numValues=2;
    else
      m_numValues=1;
  }
}




FM_CountersValueTab::FM_CountersValueTab()
:FF_ModTabPage()
,m_mainPage(NULL)
,m_counterType(NULL)
,m_valueTree(NULL)
,m_isEditing(false)
{
}



FM_CountersValueTab::~FM_CountersValueTab() {
  AE_CounterType_free(m_counterType);
}



void FM_CountersValueTab::init(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbT;
  FXString pgId;

  DBG_INFO(0, "Init");

  m_valueTree->addColumn(FF_CounterValueTree::COLUMN_DATE, 100);
  m_valueTree->addColumn(FF_CounterValueTree::COLUMN_VALUEX, 100);

  if (m_numValues>1)
    m_valueTree->addColumn(FF_CounterValueTree::COLUMN_VALUEY, 100);
  if (m_numValues>2)
    m_valueTree->addColumn(FF_CounterValueTree::COLUMN_VALUEZ, 100);

  m_valueTree->addColumn(FF_CounterValueTree::COLUMN_DIFFDAYS, 100);
  m_valueTree->addColumn(FF_CounterValueTree::COLUMN_DIFFX, 100);
  if (m_numValues>1)
    m_valueTree->addColumn(FF_CounterValueTree::COLUMN_DIFFY, 100);
  if (m_numValues>2)
    m_valueTree->addColumn(FF_CounterValueTree::COLUMN_DIFFZ, 100);

  m_valueTree->addColumn(FF_CounterValueTree::COLUMN_XPERDAY, 100);
  m_valueTree->addColumn(FF_CounterValueTree::COLUMN_XPERMONTH, 100);
  m_valueTree->addColumn(FF_CounterValueTree::COLUMN_XPERYEAR, 100);

  if (m_numValues>1) {
    m_valueTree->addColumn(FF_CounterValueTree::COLUMN_YPERDAY, 100);
    m_valueTree->addColumn(FF_CounterValueTree::COLUMN_YPERMONTH, 100);
    m_valueTree->addColumn(FF_CounterValueTree::COLUMN_YPERYEAR, 100);
  }

  if (m_numValues>2) {
    m_valueTree->addColumn(FF_CounterValueTree::COLUMN_ZPERDAY, 100);
    m_valueTree->addColumn(FF_CounterValueTree::COLUMN_ZPERMONTH, 100);
    m_valueTree->addColumn(FF_CounterValueTree::COLUMN_ZPERYEAR, 100);
  }

  m_valueTree->addColumn(FF_CounterValueTree::COLUMN_TOTALXPERDAY, 100);
  m_valueTree->addColumn(FF_CounterValueTree::COLUMN_TOTALXPERMONTH, 100);
  m_valueTree->addColumn(FF_CounterValueTree::COLUMN_TOTALXPERYEAR, 100);

  if (m_numValues>1) {
    m_valueTree->addColumn(FF_CounterValueTree::COLUMN_TOTALYPERDAY, 100);
    m_valueTree->addColumn(FF_CounterValueTree::COLUMN_TOTALYPERMONTH, 100);
    m_valueTree->addColumn(FF_CounterValueTree::COLUMN_TOTALYPERYEAR, 100);
  }

  if (m_numValues>2) {
    m_valueTree->addColumn(FF_CounterValueTree::COLUMN_TOTALZPERDAY, 100);
    m_valueTree->addColumn(FF_CounterValueTree::COLUMN_TOTALZPERMONTH, 100);
    m_valueTree->addColumn(FF_CounterValueTree::COLUMN_TOTALZPERYEAR, 100);
  }


  pgId="valueTab";
  pgId+=getPageId();
  dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST, pgId.text());
  if (dbT) {
    int i;
    int sortByColumn;
    int sortUp;

    sortByColumn=GWEN_DB_GetIntValue(dbT, "sortByColumn", 0, -1);
    sortUp=GWEN_DB_GetIntValue(dbT, "sortUp", 0, 0);

    for (i=0; i<m_valueTree->getNumHeaders(); i++) {
      int j=GWEN_DB_GetIntValue(dbT, "columns", i, m_valueTree->getHeaderSize(i));
      m_valueTree->setHeaderSize(i, j);
      if (i==sortByColumn) {
        if (sortUp)
          m_valueTree->setHeaderArrowDir(i, TRUE);
        else
          m_valueTree->setHeaderArrowDir(i, FALSE);
      }
    }
  }
  reload();
}



void FM_CountersValueTab::fini(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbT;
  FXString pgId;
  int i;

  DBG_INFO(0, "Fini");

  pgId="valueTab";
  pgId+=getPageId();
  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, pgId.text());
  GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", -1);
  for (i=0; i<m_valueTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_DEFAULT, "columns",
                        m_valueTree->getHeaderSize(i));
    b=m_valueTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
        GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
        GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }
}



void FM_CountersValueTab::create(){
  FF_ModTabPage::create();
  m_valueTree->create();
}



long FM_CountersValueTab::onCmdAddValue(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_COUNTER_VALUE *tmpl;

    tmpl=AE_CounterValue_new();
    AE_CounterValue_SetCounterId(tmpl, AE_CounterType_GetId(m_counterType));

    if (FF_EditCounterValue::createCounterValues(app, this, I18N("Add Value"), tmpl)) {
      DBG_ERROR(0, "Added value(s)");
    }
    AE_CounterValue_free(tmpl);
  }

  return 1;
}



long FM_CountersValueTab::onCmdEditValue(FXObject*, FXSelector, void*) {
  AE_COUNTER_VALUE *cv;

  cv=m_valueTree->getCurrentObject();
  if (cv) {
    FF_App *app;
    AE_BOOK *b;

    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      if (FF_EditCounterValue::editCounterValue(app, this, I18N("Edit Value"), cv)) {
        int rv;

        rv=AE_Book_WriteCounterValue(b, AE_Book_TableType_CounterValue, cv, 1);
        if (rv<0) {
          DBG_ERROR(0, "Unable to write counter value back (%d)", rv);
        }
        app->signalDbReload(AE_Book_TableType_CounterValue);
      }
    }
  }

  return 1;
}



long FM_CountersValueTab::onCmdDelValue(FXObject*, FXSelector, void*) {
#if 0
  AE_COUNTER_VALUE *cv;

  cv=m_valueTree->getCurrentValue();
  if (veh) {
    FF_App *app;
    AQDB_ID id;

    app=dynamic_cast<FF_App*>(getApp());
    id=AE_CounterValue_GetId(cv);
    if (id)
      app->deleteValue(id);
    app->signalDbReload(AE_Book_TableType_CounterValue);
  }
#else
# warning Need to implemente this!
#endif

  return 1;
}



long FM_CountersValueTab::onTreeChanged(FXObject*, FXSelector, void* ptr) {
  if (ptr) {
    FF_GroupTreeItem *item=dynamic_cast<FF_GroupTreeItem*>((FXFoldingItem*) ptr);
    if (item) {
      if (item->isDataItem()) {
        //FF_CounterValueTreeItem *baItem=dynamic_cast<FF_CounterValueTreeItem*>(item);
        //AE_COUNTER_TYPE *veh=baItem->getVehicle();

      }
    }
  }
  return 1;
}



long FM_CountersValueTab::onTreeCommand(FXObject*, FXSelector, void* ptr) {
  if (ptr) {
    FF_GroupTreeItem *item=dynamic_cast<FF_GroupTreeItem*>((FXFoldingItem*) ptr);
    if (item && item->isDataItem()) {
      FF_CounterValueTreeItem *baItem=dynamic_cast<FF_CounterValueTreeItem*>(item);
      if (baItem) {
        AE_COUNTER_VALUE *cv;

        cv=baItem->getObject();
        if (cv) {
          FF_App *app;
          AE_BOOK *b;
      
          app=dynamic_cast<FF_App*>(getApp());
          b=app->getBook();
          if (b) {
            if (FF_EditCounterValue::editCounterValue(app, this, I18N("Edit Value"), cv)) {
              int rv;

              rv=AE_Book_WriteCounterValue(b, AE_Book_TableType_CounterValue, cv, 1);
              if (rv<0) {
                DBG_ERROR(0, "Unable to write counter value back (%d)", rv);
              }
              app->signalDbReload(AE_Book_TableType_CounterValue);
            }
          }
        }
      }
    }
  }
  return 1;
}



long FM_CountersValueTab::onTreeRightBtnPress(FXObject*, FXSelector, void* ptr) {
  FXEvent* event=(FXEvent*)ptr;
  AE_COUNTER_VALUE *cv;

  cv=m_valueTree->getCurrentObject();
  if (!event->moved){
    FXMenuPane pmenu(this);
    FXMenuCommand *c;

    new FXMenuCommand(&pmenu, I18N("Add Value"), NULL, this, ID_BUTTON_ADDVALUE);

    new FXMenuSeparator(&pmenu);

    c=new FXMenuCommand(&pmenu, I18N("Edit Value"), NULL, this, ID_BUTTON_EDITVALUE);
    if (!cv) c->disable();

    c=new FXMenuCommand(&pmenu, I18N("Delete Value"), NULL, this, ID_BUTTON_DELVALUE);
    if (!cv) c->disable();

    new FXMenuSeparator(&pmenu);
    new FXMenuCommand(&pmenu, I18N("Back"), NULL, &pmenu, ID_UNPOST);

    pmenu.create();
    pmenu.popup(NULL, event->root_x, event->root_y);
    getApp()->runModalWhileShown(&pmenu);
  }

  return 1;
}



void FM_CountersValueTab::dbOpened(AE_BOOK *bk) {
  dbReload(AE_Book_TableType_None);
}



void FM_CountersValueTab::dbClosing() {
  m_valueTree->clearItems();
}



void FM_CountersValueTab::dbClosed() {
}



void FM_CountersValueTab::reload() {
  FF_App *app;
  AE_BOOK *b;
  AE_COUNTER_VALUE *cv;
  AQDB_ID currentValueId=0;

  cv=m_valueTree->getCurrentObject();
  if (cv)
    currentValueId=AE_CounterValue_GetId(cv);

  m_valueTree->clearItems();
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    int rv;
    AE_COUNTER_VALUE_LIST *rel;
    char qexpr[64];

    snprintf(qexpr, sizeof(qexpr)-1, "$counterId==%lld", (unsigned long long int) AE_CounterType_GetId(m_counterType));
    qexpr[sizeof(qexpr)-1]=0;

    /* read counter values */
    rel=AE_CounterValue_List_new();
    rv=AE_Book_QueryCounterValueList(b, AE_Book_TableType_CounterValue, 0, qexpr, rel);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }
    else {
      AE_COUNTER_VALUE *cv;

      /* sort list by date for further handling */
      AE_CounterValue_List_SortByDate(rel, 1);

      /* calculate volatile fields */
      cv=AE_CounterValue_List_First(rel);
      if (cv) {
        AE_COUNTER_VALUE *lastValue=NULL;
        AE_COUNTER_VALUE *firstValue=NULL;

        while(cv) {
          AE_CounterValues_CalculateTotalValues(cv, firstValue);
          AE_CounterValues_CalculateDiffValues(cv, lastValue);

          lastValue=cv;
          if (firstValue==NULL)
            firstValue=cv;
          cv=AE_CounterValue_List_Next(cv);
        }
      }


      while ( (cv=AE_CounterValue_List_First(rel)) ) {
        AE_CounterValue_List_Del(cv);

        /* add counter value */
        m_valueTree->addObject(cv, false);
      }
    }
    AE_CounterValue_List_free(rel);

    m_valueTree->sort();
#if 0
    for (i=0; i<m_valueTree->getNumHeaders(); i++) {
      FXbool b;

      b=m_valueTree->getHeaderArrowDir(i);
      if (b!=MAYBE) {
        if (b==TRUE)
          m_valueTree->sortByColumn(i, true);
        else
          m_valueTree->sortByColumn(i, false);
        break;
      }
    }
#endif

    m_valueTree->expandItems();
  }

  if (currentValueId!=0) {
    FF_CounterValueTreeItem *item=m_valueTree->findObjectItem(currentValueId);
    if (item) {
      item->setSelected(true);
      m_valueTree->makeItemVisible(item);
      m_valueTree->setCurrentItem(item);
    }
    else {
      DBG_ERROR(0, "Item not found");
    }
  }
}



void FM_CountersValueTab::dbReload(AE_BOOK_TABLE_TYPE tt) {
  if (tt==AE_Book_TableType_CounterValue ||
      tt==AE_Book_TableType_None) {
    reload();
  }
  else if (tt==AE_Book_TableType_CounterType) {
    /* TODO: reload counter type */
  }
}



void FM_CountersValueTab::reset() {
}



