/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2018 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif



#include "ff_bankingjobtree.hpp"

#include <fx.h>

#include <aqbanking/banking.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>
#include <gwenhywfar/gwentime.h>
#include <gwenhywfar/text.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FF_BankingJobTreeItem::FF_BankingJobTreeItem(AB_TRANSACTION *j, FXIcon* oi, FXIcon* ci, void* ptr)
:FF_GroupTreeItem(oi, ci, ptr)
,m_bankingJob(j) {
  FXString t;
  const char *s;
  char numbuf[32];

  AB_Transaction_Attach(j);

  /* job number */
  snprintf(numbuf, sizeof(numbuf)-1, "%d", AB_Transaction_GetUniqueId(j));
  numbuf[sizeof(numbuf)-1]=0;
  t+=numbuf;
  t+="\t";

  /* job type */
  s=NULL;
  switch(AB_Transaction_GetCommand(j)) {
  case AB_Transaction_CommandUnknown:                 s=I18N("unknown"); break;
  case AB_Transaction_CommandNone:                    s=I18N("none"); break;
  case AB_Transaction_CommandGetBalance:              s=I18N("Get Balance"); break;
  case AB_Transaction_CommandGetTransactions:         s=I18N("Get Transactions"); break;
  case AB_Transaction_CommandGetStandingOrders:       s=I18N("Get StandingOrders"); break;
  case AB_Transaction_CommandGetDatedTransfers:       s=I18N("Get DatedTransfers"); break;
  case AB_Transaction_CommandSepaGetStandingOrders:   s=I18N("Get Standing Orders (SEPA)"); break;
  case AB_Transaction_CommandLoadCellPhone:           s=I18N("Load Cellphone"); break;
  case AB_Transaction_CommandGetEStatements:          s=I18N("Get Electronic Statements"); break;
  case AB_Transaction_CommandTransfer:                s=I18N("Transfer"); break;
  case AB_Transaction_CommandDebitNote:               s=I18N("Debitnote"); break;
  case AB_Transaction_CommandCreateStandingOrder:     s=I18N("Create Standing Order"); break;
  case AB_Transaction_CommandModifyStandingOrder:     s=I18N("Modify Standing Order"); break;
  case AB_Transaction_CommandDeleteStandingOrder:     s=I18N("Delete Standing Order"); break;
  case AB_Transaction_CommandCreateDatedTransfer:     s=I18N("Create Dated Transfer"); break;
  case AB_Transaction_CommandModifyDatedTransfer:     s=I18N("Modify Dated Transfer"); break;
  case AB_Transaction_CommandDeleteDatedTransfer:     s=I18N("Delete Dated Transfer"); break;
  case AB_Transaction_CommandInternalTransfer:        s=I18N("Internal Transfer"); break;
  case AB_Transaction_CommandSepaTransfer:            s=I18N("Transfer (SEPA)"); break;
  case AB_Transaction_CommandSepaDebitNote:           s=I18N("Debitnote (SEPA)"); break;
  case AB_Transaction_CommandSepaFlashDebitNote:      s=I18N("Flash Debitnote (SEPA)"); break;
  case AB_Transaction_CommandSepaCreateStandingOrder: s=I18N("Create Standing Order (SEPA)"); break;
  case AB_Transaction_CommandSepaModifyStandingOrder: s=I18N("Modify Standing Order (SEPA)"); break;
  case AB_Transaction_CommandSepaDeleteStandingOrder: s=I18N("Delete Standing Order (SEPA)"); break;
  case AB_Transaction_CommandSepaCreateDatedTransfer: s=I18N("Create Dated Transfer (SEPA)"); break;
  case AB_Transaction_CommandSepaModifyDatedTransfer: s=I18N("Modify Dated Transfer (SEPA)"); break;
  case AB_Transaction_CommandSepaDeleteDatedTransfer: s=I18N("Delete Dated Transfer (SEPA)"); break;
  case AB_Transaction_CommandSepaInternalTransfer:    s=I18N("Internal Transfer (SEPA)"); break;
  }
  if (s)
    t+=s;
  t+="\t";

  /* details */
  t+=createJobDetails(j);
  t+="\t";

  /* bank name */
  s=AB_Transaction_GetLocalBic(j);
  if (s==NULL)
    s=AB_Transaction_GetLocalBankCode(j);
  if (s)
    t+=FXString(s);
  t+="\t";

  /* account name */
  s=AB_Transaction_GetLocalIban(j);
  if (s==NULL)
    s=AB_Transaction_GetLocalAccountNumber(j);
  if (s)
    t+=FXString(s);
  t+="\t";


  /* status */
  switch(AB_Transaction_GetStatus(j)) {
  case AB_Transaction_StatusUnknown:            s=I18N("JobStatus|unknown"); break;
  case AB_Transaction_StatusNone:               s=I18N("JobStatus|new"); break;
  case AB_Transaction_StatusAccepted:           s=I18N("JobStatus|accepted"); break;
  case AB_Transaction_StatusRejected:           s=I18N("JobStatus|rejected"); break;
  case AB_Transaction_StatusPending:            s=I18N("JobStatus|pending"); break;
  case AB_Transaction_StatusSending:            s=I18N("JobStatus|sending"); break;
  case AB_Transaction_StatusAutoReconciled:     s=I18N("JobStatus|reconciled (auto)"); break;
  case AB_Transaction_StatusManuallyReconciled: s=I18N("JobStatus|reconciled (manually)"); break;
  case AB_Transaction_StatusRevoked:            s=I18N("JobStatus|revoked"); break;
  case AB_Transaction_StatusAborted:            s=I18N("JobStatus|aborted"); break;
  case AB_Transaction_StatusEnqueued:           s=I18N("JobStatus|enqueued"); break;
  case AB_Transaction_StatusError:              s=I18N("JobStatus|error"); break;
  case AB_Transaction_StatusSent:               s=I18N("JobStatus|sent"); break;
  }
  if (s)
    t+=s;
  t+="\t";

  setText(t);
}



FF_BankingJobTreeItem::~FF_BankingJobTreeItem() {
  if (m_isDataItem && m_bankingJob) {
    AB_Transaction_free(m_bankingJob);
    m_bankingJob=(AB_TRANSACTION*) -1;
  }
}



FXString FF_BankingJobTreeItem::createJobDetails(const AB_TRANSACTION *j) {
  FXString result;

  switch(AB_Transaction_GetCommand(j)) {
  case AB_Transaction_CommandGetBalance:
    break;
  case AB_Transaction_CommandGetTransactions: {
    const GWEN_DATE *da1=AB_Transaction_GetFirstDate(j);
    const GWEN_DATE *da2=AB_Transaction_GetLastDate(j);

    if (da1 && da2) {
      GWEN_BUFFER *tbuf;
      FXString fromDate;
      FXString toDate;

      tbuf=GWEN_Buffer_new(0, 256, 0, 1);
      GWEN_Date_toStringWithTemplate(da1, I18N("YYYY/MM/DD"), tbuf);
      fromDate=FXString(GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_Reset(tbuf);

      GWEN_Date_toStringWithTemplate(da2, I18N("YYYY/MM/DD"), tbuf);
      toDate=FXString(GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_free(tbuf);

      result=FXStringFormat("Get statements from %s to %s",
			    fromDate.text(), toDate.text());
    }
    else if (da1) {
      GWEN_BUFFER *tbuf;
      FXString fromDate;

      tbuf=GWEN_Buffer_new(0, 256, 0, 1);
      GWEN_Date_toStringWithTemplate(da1, I18N("YYYY/MM/DD"), tbuf);
      fromDate=FXString(GWEN_Buffer_GetStart(tbuf));

      result=FXStringFormat("Get statements from %s on",
			    fromDate.text());
      GWEN_Buffer_free(tbuf);
    }
    else if (da2) {
      GWEN_BUFFER *tbuf;
      FXString toDate;

      tbuf=GWEN_Buffer_new(0, 256, 0, 1);
      GWEN_Date_toStringWithTemplate(da2, I18N("YYYY/MM/DD"), tbuf);
      toDate=FXString(GWEN_Buffer_GetStart(tbuf));

      result=FXStringFormat("Get statements up until %s",
			    toDate.text());
      GWEN_Buffer_free(tbuf);
    }
    else {
      result=FXString(I18N("Get latest statements"));
    }
    break;
  }

  case AB_Transaction_CommandTransfer: {
    const char *s;
    FXString recipient;
    FXString amount;
    const AB_VALUE *v;

    s=AB_Transaction_GetRemoteName(j);
    if (s)
      recipient=FXString(s);

    if (recipient.empty()) {
      const char *s1;
      const char *s2;

      s1=AB_Transaction_GetRemoteBankCode(j);
      s2=AB_Transaction_GetRemoteAccountNumber(j);
      if (s1 && s2)
        recipient=FXString(s1)+"/"+FXString(s2);
      else if (s1)
        recipient=FXString(s1);
      else if (s2)
        recipient=FXString(s2);
      else
        recipient=FXString(I18N("Unknown recipient!"));
    }

    v=AB_Transaction_GetValue(j);
    if (v==NULL) {
      amount=FXString(I18N("Unknown amount!"));
    }
    else {
      GWEN_BUFFER *tbuf;

      tbuf=GWEN_Buffer_new(0, 32, 0, 1);
      AB_Value_toHumanReadableString(v, tbuf, 2, 0);
      amount=FXString(GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_free(tbuf);
    }

    result=FXStringFormat(I18N("Transfer %s to %s"), amount.text(), recipient.text());
    break;
  }

  case AB_Transaction_CommandSepaTransfer: {
    const char *s;
    FXString recipient;
    FXString amount;
    const AB_VALUE *v;

    s=AB_Transaction_GetRemoteName(j);
    if (s)
      recipient=FXString(s);

    if (recipient.empty()) {
      const char *s1;
      const char *s2;

      s1=AB_Transaction_GetRemoteBic(j);
      s2=AB_Transaction_GetRemoteIban(j);
      if (s1 && s2)
        recipient=FXString(s1)+"/"+FXString(s2);
      else if (s1)
        recipient=FXString(s1);
      else if (s2)
        recipient=FXString(s2);
      else
        recipient=FXString(I18N("Unknown recipient!"));
    }

    v=AB_Transaction_GetValue(j);
    if (v==NULL) {
      amount=FXString(I18N("Unknown amount!"));
    }
    else {
      GWEN_BUFFER *tbuf;

      tbuf=GWEN_Buffer_new(0, 32, 0, 1);
      AB_Value_toHumanReadableString(v, tbuf, 2, 0);
      amount=FXString(GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_free(tbuf);
    }

    result=FXStringFormat(I18N("Transfer %s to %s (SEPA)"), amount.text(), recipient.text());
    break;
  }

  case AB_Transaction_CommandLoadCellPhone: {
#if 0
    const char *sPhoneNumber=NULL;
    const char *sProductName=NULL;
    FXString recipient;
    FXString amount;
    const AB_VALUE *v;
    const AB_CELLPHONE_PRODUCT *p;

    sPhoneNumber=AB_JobLoadCellPhone_GetPhoneNumber(j);
    if (sPhoneNumber && *sPhoneNumber==0)
      sPhoneNumber=NULL;
    p=AB_JobLoadCellPhone_GetCellPhoneProduct(j);
    if (p)
      sProductName=AB_CellPhoneProduct_GetProductName(p);
    if (sProductName && *sProductName==0)
      sProductName=NULL;

    if (sPhoneNumber && sProductName)
      recipient=FXString(sPhoneNumber)+" ("+FXString(sProductName)+")";
    else if (sPhoneNumber)
      recipient=FXString(sPhoneNumber);
    else if (sProductName)
      recipient=FXString(sProductName);
    else
      recipient=FXString(I18N("Unknown recipient!"));

    v=AB_JobLoadCellPhone_GetValue(j);
    if (v==NULL) {
      amount=FXString(I18N("Unknown amount!"));
    }
    else {
      GWEN_BUFFER *tbuf;

      tbuf=GWEN_Buffer_new(0, 32, 0, 1);
      AB_Value_toHumanReadableString(v, tbuf, 2);
      amount=FXString(GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_free(tbuf);
    }

    result=FXStringFormat(I18N("Load cell phone %s with %s"), recipient.text(), amount.text());
    break;
#else
    result=FXString("Load cell phone");
    break;
#endif
  }

  case AB_Transaction_CommandUnknown:
  case AB_Transaction_CommandNone:
  case AB_Transaction_CommandGetStandingOrders:
  case AB_Transaction_CommandGetDatedTransfers:
  case AB_Transaction_CommandSepaGetStandingOrders:
  case AB_Transaction_CommandGetEStatements:
  case AB_Transaction_CommandDebitNote:
  case AB_Transaction_CommandCreateStandingOrder:
  case AB_Transaction_CommandModifyStandingOrder:
  case AB_Transaction_CommandDeleteStandingOrder:
  case AB_Transaction_CommandCreateDatedTransfer:
  case AB_Transaction_CommandModifyDatedTransfer:
  case AB_Transaction_CommandDeleteDatedTransfer:
  case AB_Transaction_CommandInternalTransfer:
  case AB_Transaction_CommandSepaDebitNote:
  case AB_Transaction_CommandSepaFlashDebitNote:
  case AB_Transaction_CommandSepaCreateStandingOrder:
  case AB_Transaction_CommandSepaModifyStandingOrder:
  case AB_Transaction_CommandSepaDeleteStandingOrder:
  case AB_Transaction_CommandSepaCreateDatedTransfer:
  case AB_Transaction_CommandSepaModifyDatedTransfer:
  case AB_Transaction_CommandSepaDeleteDatedTransfer:
  case AB_Transaction_CommandSepaInternalTransfer:
    break;
  } /* switch */
  return result;
}




FF_BankingJobTree::FF_BankingJobTree(FXComposite *p,
				     FXObject* tgt, FXSelector sel,
				     FXuint opts,
				     FXint x, FXint y, FXint w, FXint h)
:FF_GroupTree(p, tgt, sel, opts, x, y, w, h) {
  int idx=0;

  appendHeader(I18N("Job Id"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("Job Type"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("Details"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("Bank"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("Account"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("Status"));
  setHeaderSize(idx++, 100);
}



FF_BankingJobTree::~FF_BankingJobTree() {
}



FF_BankingJobTreeItem *FF_BankingJobTree::findBankingJobItem(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem;

    grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem) {
      FF_BankingJobTreeItem *res=findBankingJobItem(grItem, id);
      if (res!=NULL)
        return res;
    }
    item=item->getBelow();
  }

  return NULL;
}



FF_BankingJobTreeItem *FF_BankingJobTree::findBankingJobItem(FF_GroupTreeItem *grItem, AQDB_ID id) {
  if (grItem->isDataItem()) {
    FF_BankingJobTreeItem *jItem=dynamic_cast<FF_BankingJobTreeItem*>(grItem);
    if (jItem) {
      AB_TRANSACTION *j=jItem->getBankingJob();
      if (AB_Transaction_GetUniqueId(j)==id)
        return jItem;
    }
  }
  else {
    FXFoldingItem *item;

    item=grItem->getFirst();
    while(item) {
      grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem) {
        FF_BankingJobTreeItem *res=findBankingJobItem(grItem, id);
        if (res!=NULL)
          return res;
      }
      item=item->getNext();
    }
  }

  return NULL;
}



void FF_BankingJobTree::addJob(AB_TRANSACTION *j) {
  FF_GroupTreeItem *itemFather=NULL;
  AQDB_ID gid;
  FF_BankingJobTreeItem *item;

  gid=AB_Transaction_GetUniqueAccountId(j);
  if (gid) {
    itemFather=findGroupItem(gid);
    if (itemFather==NULL) {
      DBG_ERROR(AE_LOGDOMAIN, "Group for bank account not found, reassigning to root");
    }
  }

  item=new FF_BankingJobTreeItem(j,
				 /*m_iconOpen, m_iconClosed*/
				 NULL, NULL); /* no icons here */
  appendItem(itemFather, item, false);
}



FF_BankingJobTree::FF_BankingJobTree(): FF_GroupTree() {
}



void FF_BankingJobTree::create() {
  FF_GroupTree::create();
}



AB_TRANSACTION *FF_BankingJobTree::getCurrentBankingJob() {
  FXFoldingItem *item;

  item=getCurrentItem();
  if (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_BankingJobTreeItem *jItem;

      jItem=dynamic_cast<FF_BankingJobTreeItem*>(grItem);
      if (jItem)
        return jItem->getBankingJob();
    }
  }

  return NULL;
}



AB_TRANSACTION *FF_BankingJobTree::getFirstSelectedBankingJob() {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_BankingJobTreeItem *jItem;

        jItem=dynamic_cast<FF_BankingJobTreeItem*>(grItem);
        if (jItem)
          return jItem->getBankingJob();
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_BankingJobTree::getSelectedBankingJobs(AB_TRANSACTION_LIST2 *al) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_BankingJobTreeItem *jItem=dynamic_cast<FF_BankingJobTreeItem*>(grItem);
        if (jItem)
          AB_Transaction_List2_PushBack(al, jItem->getBankingJob());
      }
    }
    item=item->getBelow();
  }
}



void FF_BankingJobTree::selectBankingJob(AQDB_ID id, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_BankingJobTreeItem *jItem;

      jItem=dynamic_cast<FF_BankingJobTreeItem*>(grItem);
      if (jItem) {
        AB_TRANSACTION *j;

        j=jItem->getBankingJob();
        if (AB_Transaction_GetUniqueId(j)==id) {
          jItem->setSelected(b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_BankingJobTree::makeBankingJobVisible(AQDB_ID id) {
  FXFoldingItem *item;

  item=findBankingJobItem(id);
  if (item)
    makeItemVisible(item);
}






