/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2014 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif



#include "ff_countertypetree.hpp"

#include <aqfinance/engine/book/ae_utils.h>

#include <fx.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>
#include <gwenhywfar/gwentime.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)


#define SIDE_SPACING 4
#define ICON_SPACING 4



FF_CounterTypeTreeItem::FF_CounterTypeTreeItem(AE_COUNTER_TYPE *st, void* ptr)
:FF_GroupTreeItem(NULL, NULL, ptr)
,m_object(st)
{
}



FF_CounterTypeTreeItem::~FF_CounterTypeTreeItem() {
  if (m_isDataItem && m_object) {
    AE_CounterType_free(m_object);
    m_object=(AE_COUNTER_TYPE*) -1;
  }
}



FXString FF_CounterTypeTreeItem::getItemText(const FXXCheckTree* list) const {
  const FF_CounterTypeTree *tree;
  FXString t;

  tree=dynamic_cast<const FF_CounterTypeTree*>(list);
  if (tree) {
    int i;

    for (i=0; i<tree->m_columns; i++) {
      const char *s;

      switch(tree->m_columnTypes[i]) {

      case FF_CounterTypeTree::COLUMN_NAME:
        s=AE_CounterType_GetName(m_object);
        if (s)
          t+=FXString(s);
        break;

      case FF_CounterTypeTree::COLUMN_DESCRIPTION:
        s=AE_CounterType_GetDescription(m_object);
        if (s)
          t+=FXString(s);
        break;

      case FF_CounterTypeTree::COLUMN_XLABEL:
        s=AE_CounterType_GetXLabel(m_object);
        if (s)
          t+=FXString(s);
        break;

      case FF_CounterTypeTree::COLUMN_YLABEL:
        s=AE_CounterType_GetYLabel(m_object);
        if (s)
          t+=FXString(s);
        break;

      case FF_CounterTypeTree::COLUMN_ZLABEL:
        s=AE_CounterType_GetZLabel(m_object);
        if (s)
          t+=FXString(s);
        break;

      default:
        break;
      }
      t+="\t";
    } /* for */
  }

  return t;
}









FF_CounterTypeTree::FF_CounterTypeTree(FXComposite *p,
                                       FXObject* tgt, FXSelector sel,
                                       FXuint opts,
                                       FXint x, FXint y, FXint w, FXint h)
:FF_GroupTree(p, tgt, sel, opts, x, y, w, h)
,m_columns(0)
{
}



void FF_CounterTypeTree::addColumn(int c, int w) {
  if (m_columns<COLUMN_MAX) {
    switch(c) {
    case COLUMN_NAME:
      appendHeader(I18N("AE_CounterType|Name"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    case COLUMN_DESCRIPTION:
      appendHeader(I18N("AE_CounterType|Description"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    case COLUMN_XLABEL:
      appendHeader(I18N("AE_CounterType|X-Label"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    case COLUMN_YLABEL:
      appendHeader(I18N("AE_CounterType|Y-Label"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    case COLUMN_ZLABEL:
      appendHeader(I18N("AE_CounterType|Y-Label"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    default:
      break;
    }
  }
}



void FF_CounterTypeTree::addColumn(const char *colName, int c, int w) {
  assert(colName);
  if (m_columns<COLUMN_MAX) {
    switch(c) {
    case COLUMN_NAME:
    case COLUMN_DESCRIPTION:
    case COLUMN_XLABEL:
    case COLUMN_YLABEL:
    case COLUMN_ZLABEL:
      appendHeader(colName);
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    default:
      break;
    }
  }
}



FF_CounterTypeTree::~FF_CounterTypeTree() {
}



FF_CounterTypeTreeItem *FF_CounterTypeTree::findObjectItem(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem;

    grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem) {
      FF_CounterTypeTreeItem *res=findObjectItem(grItem, id);
      if (res!=NULL)
        return res;
    }
    item=item->getNext();
  }

  return NULL;
}



FF_CounterTypeTreeItem *FF_CounterTypeTree::findObjectItem(FF_GroupTreeItem *grItem, AQDB_ID id) {
  if (grItem->isDataItem()) {
    FF_CounterTypeTreeItem *stItem=dynamic_cast<FF_CounterTypeTreeItem*>(grItem);
    if (stItem) {
      AE_COUNTER_TYPE *st=stItem->getObject();
      if (AE_CounterType_GetId(st)==id)
        return stItem;
    }
  }
  else {
    FXFoldingItem *item;

    item=grItem->getFirst();
    while(item) {
      grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem) {
        FF_CounterTypeTreeItem *res=findObjectItem(grItem, id);
        if (res!=NULL)
          return res;
      }
      item=item->getNext();
    }
  }

  return NULL;
}



FF_CounterTypeTreeItem *FF_CounterTypeTree::findObjectItem(const AE_COUNTER_TYPE *qba) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_CounterTypeTreeItem *baItem;

      baItem=dynamic_cast<FF_CounterTypeTreeItem*>(grItem);
      if (baItem) {
        AE_COUNTER_TYPE *sta;

	sta=baItem->getObject();
	if (sta==qba)
          return baItem;
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_CounterTypeTree::addObject(AE_COUNTER_TYPE *st, bool direct, int useColor) {
  FF_GroupTreeItem *itemFather=NULL;
  FF_CounterTypeTreeItem *item;

  if (!direct) {
    AQDB_ID gid;

    gid=AE_CounterType_GetGroupId(st);
    if (gid) {
      itemFather=findGroupItem(gid);
      if (itemFather==NULL) {
        DBG_INFO(AE_LOGDOMAIN, "Group for statement not found, reassigning to root");
      }
    }
  }

  item=new FF_CounterTypeTreeItem(st);
  /* maybe colorize */
  switch(useColor) {
  case FF_CounterTypeTree::USECOLOR_NOTICE:
    item->setUseOwnColors(true);
    item->setBackColor(FXRGB(247, 248, 96));
    item->setTextColor(FXRGB(0, 0, 0));
    break;
  case FF_CounterTypeTree::USECOLOR_WARNING:
    item->setUseOwnColors(true);
    item->setBackColor(FXRGB(255, 127, 0));
    item->setTextColor(FXRGB(255, 255, 255));
    break;

  case FF_CounterTypeTree::USECOLOR_NORMAL:
  default:
    item->setUseOwnColors(false);
    break;
  }

  appendItem(itemFather, item, false);
}



FF_CounterTypeTree::FF_CounterTypeTree(): FF_GroupTree() {
}



void FF_CounterTypeTree::create() {
  FF_GroupTree::create();
}



AE_COUNTER_TYPE *FF_CounterTypeTree::getCurrentObject() {
  FXFoldingItem *item;

  item=getCurrentItem();
  if (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_CounterTypeTreeItem *stItem;

      stItem=dynamic_cast<FF_CounterTypeTreeItem*>(grItem);
      if (stItem)
        return stItem->getObject();
    }
  }

  return NULL;
}



AE_COUNTER_TYPE *FF_CounterTypeTree::getFirstSelectedObject() {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_CounterTypeTreeItem *stItem;

        stItem=dynamic_cast<FF_CounterTypeTreeItem*>(grItem);
        if (stItem)
          return stItem->getObject();
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_CounterTypeTree::addObjects(AE_COUNTER_TYPE_LIST2 *sl, int copyMode, bool direct) {
  AE_COUNTER_TYPE_LIST2_ITERATOR *it;

  it=AE_CounterType_List2_First(sl);
  if (it) {
    AE_COUNTER_TYPE *st;

    st=AE_CounterType_List2Iterator_Data(it);
    switch(copyMode) {
    case COPYMODE_COPY:
      while(st) {
        addObject(AE_CounterType_dup(st), direct, USECOLOR_NORMAL);
        st=AE_CounterType_List2Iterator_Next(it);
      }
      break;

    case COPYMODE_ATTACH:
      while(st) {
        AE_CounterType_Attach(st);
        addObject(st, direct, USECOLOR_NORMAL);
        st=AE_CounterType_List2Iterator_Next(it);
      }
      break;

    case COPYMODE_NONE:
    default:
      while(st) {
        addObject(st, direct, USECOLOR_NORMAL);
        st=AE_CounterType_List2Iterator_Next(it);
      }
      break;
    }
    AE_CounterType_List2Iterator_free(it);
  }
}



void FF_CounterTypeTree::getSelectedObjects(AE_COUNTER_TYPE_LIST2 *sl, int copyMode) {
  FXFoldingItem *item;

  switch(copyMode) {
  case COPYMODE_COPY:
    item=getFirstItem();
    while (item) {
      if (item->isSelected()) {
        FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
        if (grItem && (grItem->isDataItem())) {
          FF_CounterTypeTreeItem *stItem=dynamic_cast<FF_CounterTypeTreeItem*>(grItem);
          if (stItem)
            AE_CounterType_List2_PushBack(sl, AE_CounterType_dup(stItem->getObject()));
        }
      }
      item=item->getBelow();
    }
    break;

  case COPYMODE_ATTACH:
    item=getFirstItem();
    while (item) {
      if (item->isSelected()) {
        FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
        if (grItem && (grItem->isDataItem())) {
          FF_CounterTypeTreeItem *stItem=dynamic_cast<FF_CounterTypeTreeItem*>(grItem);
	  if (stItem) {
	    AE_COUNTER_TYPE *st=stItem->getObject();
            AE_CounterType_Attach(st);
            AE_CounterType_List2_PushBack(sl, st);
          }
        }
      }
      item=item->getBelow();
    }
    break;

  case COPYMODE_NONE:
  default:
    item=getFirstItem();
    while (item) {
      if (item->isSelected()) {
        FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
        if (grItem && (grItem->isDataItem())) {
          FF_CounterTypeTreeItem *stItem=dynamic_cast<FF_CounterTypeTreeItem*>(grItem);
          if (stItem)
            AE_CounterType_List2_PushBack(sl, stItem->getObject());
        }
      }
      item=item->getBelow();
    }
    break;
  }
}



void FF_CounterTypeTree::selectObject(AQDB_ID id, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_CounterTypeTreeItem *stItem;

      stItem=dynamic_cast<FF_CounterTypeTreeItem*>(grItem);
      if (stItem) {
        AE_COUNTER_TYPE *st;

        st=stItem->getObject();
        if (AE_CounterType_GetId(st)==id) {
          stItem->setSelected(b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_CounterTypeTree::selectObject(const AE_COUNTER_TYPE *fst, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_CounterTypeTreeItem *stItem;

      stItem=dynamic_cast<FF_CounterTypeTreeItem*>(grItem);
      if (stItem) {
        if (stItem->getObject()==fst) {
          stItem->setSelected(b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_CounterTypeTree::makeObjectVisible(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_CounterTypeTreeItem *stItem;

      stItem=dynamic_cast<FF_CounterTypeTreeItem*>(grItem);
      if (stItem) {
        AE_COUNTER_TYPE *st;

        st=stItem->getObject();
        if (AE_CounterType_GetId(st)==id) {
          makeItemVisible(stItem);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}




GROUPTREE_SORT_FN_UP_TXT(FF_CounterTypeTree, FF_CounterTypeTreeItem, getObject, AE_COUNTER_TYPE, AE_CounterType, Name)
GROUPTREE_SORT_FN_DOWN(FF_CounterTypeTree, FF_CounterTypeTreeItem, getObject, AE_COUNTER_TYPE, AE_CounterType, Name)






void FF_CounterTypeTree::sortByColumn(int c, bool up) {
  if (c<m_columns) {
    switch(m_columnTypes[c]) {
    case COLUMN_NAME:
      setSortFunc(up?sortFunctionUpName:sortFunctionDownName);
      sortItems();
      break;

    default:
      DBG_ERROR(0, "Sort by column %d (%s) not implemented", c, up?"up":"down");
      break;
    }
  }
}



