/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2014 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif



#include "ff_countervaluetree.hpp"

#include <aqfinance/engine/book/ae_utils.h>

#include <fx.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>
#include <gwenhywfar/gwendate.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)


#define SIDE_SPACING 4
#define ICON_SPACING 4



FF_CounterValueTreeItem::FF_CounterValueTreeItem(AE_COUNTER_VALUE *st, void* ptr)
:FF_GroupTreeItem(NULL, NULL, ptr)
,m_object(st)
{
}



FF_CounterValueTreeItem::~FF_CounterValueTreeItem() {
  if (m_isDataItem && m_object) {
    AE_CounterValue_free(m_object);
    m_object=(AE_COUNTER_VALUE*) -1;
  }
}



FXString FF_CounterValueTreeItem::_valueToString(const AQDB_VALUE *v) {
  FXString t;

  if (v) {
    GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);

    t+="<right>";

    AQDB_Value_toHumanReadableString(v, tbuf, 2);
    t+=GWEN_Buffer_GetStart(tbuf);
    GWEN_Buffer_free(tbuf);
    t+="</right>";
  }

  return t;
}


FXString FF_CounterValueTreeItem::getItemText(const FXXCheckTree* list) const {
  const FF_CounterValueTree *tree;
  FXString t;

  tree=dynamic_cast<const FF_CounterValueTree*>(list);
  if (tree) {
    int i;

    for (i=0; i<tree->m_columns; i++) {
      const char *s;

      switch(tree->m_columnTypes[i]) {

      case FF_CounterValueTree::COLUMN_DATE:
        {
          const GWEN_DATE *ti;

          ti=AE_CounterValue_GetDate(m_object);
          if (ti) {
            GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);
            GWEN_Date_toStringWithTemplate(ti, I18N("YYYY/MM/DD"), tbuf);
            t+=GWEN_Buffer_GetStart(tbuf);
            GWEN_Buffer_free(tbuf);
          }
        }
        break;

      case FF_CounterValueTree::COLUMN_VALUEX:
        t+=_valueToString(AE_CounterValue_GetValueX(m_object));
        break;

      case FF_CounterValueTree::COLUMN_VALUEY:
        t+=_valueToString(AE_CounterValue_GetValueY(m_object));
        break;

      case FF_CounterValueTree::COLUMN_VALUEZ:
        t+=_valueToString(AE_CounterValue_GetValueZ(m_object));
        break;

      case FF_CounterValueTree::COLUMN_NOTE:
        s=AE_CounterValue_GetNote(m_object);
        if (s)
          t+=FXString(s);
        break;

      case FF_CounterValueTree::COLUMN_DIFFDAYS:
	{
	  char numbuf[32];

	  snprintf(numbuf, sizeof(numbuf)-1, I18N("%d"), AE_CounterValue_GetDiffDays(m_object));
	  numbuf[sizeof(numbuf)-1]=0;
          t+="<right>";
	  t+=numbuf;
          t+="</right>";
        }
        break;

      case FF_CounterValueTree::COLUMN_DIFFX:
        t+=_valueToString(AE_CounterValue_GetDiffX(m_object));
        break;

      case FF_CounterValueTree::COLUMN_DIFFY:
        t+=_valueToString(AE_CounterValue_GetDiffY(m_object));
        break;

      case FF_CounterValueTree::COLUMN_DIFFZ:
        t+=_valueToString(AE_CounterValue_GetDiffZ(m_object));
        break;

      case FF_CounterValueTree::COLUMN_XPERDAY:
        t+=_valueToString(AE_CounterValue_GetXPerDay(m_object));
        break;

      case FF_CounterValueTree::COLUMN_XPERMONTH:
        t+=_valueToString(AE_CounterValue_GetXPerMonth(m_object));
        break;

      case FF_CounterValueTree::COLUMN_XPERYEAR:
        t+=_valueToString(AE_CounterValue_GetXPerYear(m_object));
        break;

      case FF_CounterValueTree::COLUMN_YPERDAY:
        t+=_valueToString(AE_CounterValue_GetYPerDay(m_object));
        break;

      case FF_CounterValueTree::COLUMN_YPERMONTH:
        t+=_valueToString(AE_CounterValue_GetYPerMonth(m_object));
        break;

      case FF_CounterValueTree::COLUMN_YPERYEAR:
        t+=_valueToString(AE_CounterValue_GetYPerYear(m_object));
        break;

      case FF_CounterValueTree::COLUMN_ZPERDAY:
        t+=_valueToString(AE_CounterValue_GetZPerDay(m_object));
        break;

      case FF_CounterValueTree::COLUMN_ZPERMONTH:
        t+=_valueToString(AE_CounterValue_GetZPerMonth(m_object));
        break;

      case FF_CounterValueTree::COLUMN_ZPERYEAR:
        t+=_valueToString(AE_CounterValue_GetZPerYear(m_object));
        break;

      case FF_CounterValueTree::COLUMN_TOTALXPERDAY:
        t+=_valueToString(AE_CounterValue_GetTotalXPerDay(m_object));
        break;

      case FF_CounterValueTree::COLUMN_TOTALXPERMONTH:
        t+=_valueToString(AE_CounterValue_GetTotalXPerMonth(m_object));
        break;

      case FF_CounterValueTree::COLUMN_TOTALXPERYEAR:
        t+=_valueToString(AE_CounterValue_GetTotalXPerYear(m_object));
        break;

      case FF_CounterValueTree::COLUMN_TOTALYPERDAY:
        t+=_valueToString(AE_CounterValue_GetTotalYPerDay(m_object));
        break;

      case FF_CounterValueTree::COLUMN_TOTALYPERMONTH:
        t+=_valueToString(AE_CounterValue_GetTotalYPerMonth(m_object));
        break;

      case FF_CounterValueTree::COLUMN_TOTALYPERYEAR:
        t+=_valueToString(AE_CounterValue_GetTotalYPerYear(m_object));
        break;

      case FF_CounterValueTree::COLUMN_TOTALZPERDAY:
        t+=_valueToString(AE_CounterValue_GetTotalZPerDay(m_object));
        break;

      case FF_CounterValueTree::COLUMN_TOTALZPERMONTH:
        t+=_valueToString(AE_CounterValue_GetTotalZPerMonth(m_object));
        break;

      case FF_CounterValueTree::COLUMN_TOTALZPERYEAR:
        t+=_valueToString(AE_CounterValue_GetTotalZPerYear(m_object));
        break;

      default:
        break;
      }
      t+="\t";
    } /* for */
  }

  return t;
}









FF_CounterValueTree::FF_CounterValueTree(FXComposite *p,
                                         FXObject* tgt, FXSelector sel,
                                         FXuint opts,
                                         FXint x, FXint y, FXint w, FXint h)
:FF_GroupTree(p, tgt, sel, opts, x, y, w, h)
,m_columns(0)
{
}



void FF_CounterValueTree::addColumn(int c, int w) {
  if (m_columns<COLUMN_MAX) {
    switch(c) {
    case COLUMN_DATE:
      appendHeader(I18N("AE_CounterType|Date"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    case COLUMN_VALUEX:
      appendHeader(I18N("AE_CounterType|X-Value"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    case COLUMN_VALUEY:
      appendHeader(I18N("AE_CounterType|Y-Value"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    case COLUMN_VALUEZ:
      appendHeader(I18N("AE_CounterType|Z-Value"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    case COLUMN_NOTE:
      appendHeader(I18N("AE_CounterType|Note"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    case COLUMN_DIFFDAYS:
      appendHeader(I18N("AE_CounterType|Days-Difference"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    case COLUMN_DIFFX:
      appendHeader(I18N("AE_CounterType|X-Difference"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    case COLUMN_DIFFY:
      appendHeader(I18N("AE_CounterType|Y-Difference"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    case COLUMN_DIFFZ:
      appendHeader(I18N("AE_CounterType|Z-Difference"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    case COLUMN_XPERDAY:
      appendHeader(I18N("AE_CounterType|X per day"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    case COLUMN_XPERMONTH:
      appendHeader(I18N("AE_CounterType|X per month"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    case COLUMN_XPERYEAR:
      appendHeader(I18N("AE_CounterType|X per year"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;


    case COLUMN_YPERDAY:
      appendHeader(I18N("AE_CounterType|Y per day"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    case COLUMN_YPERMONTH:
      appendHeader(I18N("AE_CounterType|Y per month"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    case COLUMN_YPERYEAR:
      appendHeader(I18N("AE_CounterType|Y per year"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;


    case COLUMN_ZPERDAY:
      appendHeader(I18N("AE_CounterType|Z per day"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    case COLUMN_ZPERMONTH:
      appendHeader(I18N("AE_CounterType|Z per month"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    case COLUMN_ZPERYEAR:
      appendHeader(I18N("AE_CounterType|Z per year"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;


    case COLUMN_TOTALXPERDAY:
      appendHeader(I18N("AE_CounterType|Total X per day"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    case COLUMN_TOTALXPERMONTH:
      appendHeader(I18N("AE_CounterType|Total X per month"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    case COLUMN_TOTALXPERYEAR:
      appendHeader(I18N("AE_CounterType|Total X per year"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;


    case COLUMN_TOTALYPERDAY:
      appendHeader(I18N("AE_CounterType|Total Y per day"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    case COLUMN_TOTALYPERMONTH:
      appendHeader(I18N("AE_CounterType|Total Y per month"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    case COLUMN_TOTALYPERYEAR:
      appendHeader(I18N("AE_CounterType|Total Y per year"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;


    case COLUMN_TOTALZPERDAY:
      appendHeader(I18N("AE_CounterType|Total Z per day"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    case COLUMN_TOTALZPERMONTH:
      appendHeader(I18N("AE_CounterType|Total Z per month"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    case COLUMN_TOTALZPERYEAR:
      appendHeader(I18N("AE_CounterType|Total Z per year"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;


    default:
      break;
    }
  }
}



void FF_CounterValueTree::addColumn(const char *colName, int c, int w) {
  assert(colName);
  if (m_columns<COLUMN_MAX) {
    switch(c) {
    case COLUMN_DATE:
    case COLUMN_VALUEX:
    case COLUMN_VALUEY:
    case COLUMN_VALUEZ:
    case COLUMN_NOTE:
    case COLUMN_DIFFDAYS:
    case COLUMN_DIFFX:
    case COLUMN_DIFFY:
    case COLUMN_DIFFZ:
    case COLUMN_XPERDAY:
    case COLUMN_XPERMONTH:
    case COLUMN_XPERYEAR:
    case COLUMN_YPERDAY:
    case COLUMN_YPERMONTH:
    case COLUMN_YPERYEAR:
    case COLUMN_ZPERDAY:
    case COLUMN_ZPERMONTH:
    case COLUMN_ZPERYEAR:
    case COLUMN_TOTALXPERDAY:
    case COLUMN_TOTALXPERMONTH:
    case COLUMN_TOTALXPERYEAR:
    case COLUMN_TOTALYPERDAY:
    case COLUMN_TOTALYPERMONTH:
    case COLUMN_TOTALYPERYEAR:
    case COLUMN_TOTALZPERDAY:
    case COLUMN_TOTALZPERMONTH:
    case COLUMN_TOTALZPERYEAR:
      appendHeader(colName);
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;

    default:
      break;
    }
  }
}



FF_CounterValueTree::~FF_CounterValueTree() {
}



FF_CounterValueTreeItem *FF_CounterValueTree::findObjectItem(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem;

    grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem) {
      FF_CounterValueTreeItem *res=findObjectItem(grItem, id);
      if (res!=NULL)
        return res;
    }
    item=item->getNext();
  }

  return NULL;
}



FF_CounterValueTreeItem *FF_CounterValueTree::findObjectItem(FF_GroupTreeItem *grItem, AQDB_ID id) {
  if (grItem->isDataItem()) {
    FF_CounterValueTreeItem *stItem=dynamic_cast<FF_CounterValueTreeItem*>(grItem);
    if (stItem) {
      AE_COUNTER_VALUE *st=stItem->getObject();
      if (AE_CounterValue_GetId(st)==id)
        return stItem;
    }
  }
  else {
    FXFoldingItem *item;

    item=grItem->getFirst();
    while(item) {
      grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem) {
        FF_CounterValueTreeItem *res=findObjectItem(grItem, id);
        if (res!=NULL)
          return res;
      }
      item=item->getNext();
    }
  }

  return NULL;
}



FF_CounterValueTreeItem *FF_CounterValueTree::findObjectItem(const AE_COUNTER_VALUE *qba) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_CounterValueTreeItem *baItem;

      baItem=dynamic_cast<FF_CounterValueTreeItem*>(grItem);
      if (baItem) {
        AE_COUNTER_VALUE *sta;

	sta=baItem->getObject();
	if (sta==qba)
          return baItem;
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_CounterValueTree::addObject(AE_COUNTER_VALUE *st, int useColor) {
  FF_CounterValueTreeItem *item;

  item=new FF_CounterValueTreeItem(st);
  /* maybe colorize */
  switch(useColor) {
  case FF_CounterValueTree::USECOLOR_NOTICE:
    item->setUseOwnColors(true);
    item->setBackColor(FXRGB(247, 248, 96));
    item->setTextColor(FXRGB(0, 0, 0));
    break;
  case FF_CounterValueTree::USECOLOR_WARNING:
    item->setUseOwnColors(true);
    item->setBackColor(FXRGB(255, 127, 0));
    item->setTextColor(FXRGB(255, 255, 255));
    break;

  case FF_CounterValueTree::USECOLOR_NORMAL:
  default:
    item->setUseOwnColors(false);
    break;
  }

  appendItem(NULL, item, false);
}



FF_CounterValueTree::FF_CounterValueTree(): FF_GroupTree() {
}



void FF_CounterValueTree::create() {
  FF_GroupTree::create();
}



AE_COUNTER_VALUE *FF_CounterValueTree::getCurrentObject() {
  FXFoldingItem *item;

  item=getCurrentItem();
  if (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_CounterValueTreeItem *stItem;

      stItem=dynamic_cast<FF_CounterValueTreeItem*>(grItem);
      if (stItem)
        return stItem->getObject();
    }
  }

  return NULL;
}



AE_COUNTER_VALUE *FF_CounterValueTree::getFirstSelectedObject() {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_CounterValueTreeItem *stItem;

        stItem=dynamic_cast<FF_CounterValueTreeItem*>(grItem);
        if (stItem)
          return stItem->getObject();
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_CounterValueTree::addObjects(AE_COUNTER_VALUE_LIST2 *sl, int copyMode) {
  AE_COUNTER_VALUE_LIST2_ITERATOR *it;

  it=AE_CounterValue_List2_First(sl);
  if (it) {
    AE_COUNTER_VALUE *st;

    st=AE_CounterValue_List2Iterator_Data(it);
    switch(copyMode) {
    case COPYMODE_COPY:
      while(st) {
        addObject(AE_CounterValue_dup(st), USECOLOR_NORMAL);
        st=AE_CounterValue_List2Iterator_Next(it);
      }
      break;

    case COPYMODE_ATTACH:
      while(st) {
        AE_CounterValue_Attach(st);
        addObject(st, USECOLOR_NORMAL);
        st=AE_CounterValue_List2Iterator_Next(it);
      }
      break;

    case COPYMODE_NONE:
    default:
      while(st) {
        addObject(st, USECOLOR_NORMAL);
        st=AE_CounterValue_List2Iterator_Next(it);
      }
      break;
    }
    AE_CounterValue_List2Iterator_free(it);
  }
}



void FF_CounterValueTree::getSelectedObjects(AE_COUNTER_VALUE_LIST2 *sl, int copyMode) {
  FXFoldingItem *item;

  switch(copyMode) {
  case COPYMODE_COPY:
    item=getFirstItem();
    while (item) {
      if (item->isSelected()) {
        FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
        if (grItem && (grItem->isDataItem())) {
          FF_CounterValueTreeItem *stItem=dynamic_cast<FF_CounterValueTreeItem*>(grItem);
          if (stItem)
            AE_CounterValue_List2_PushBack(sl, AE_CounterValue_dup(stItem->getObject()));
        }
      }
      item=item->getBelow();
    }
    break;

  case COPYMODE_ATTACH:
    item=getFirstItem();
    while (item) {
      if (item->isSelected()) {
        FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
        if (grItem && (grItem->isDataItem())) {
          FF_CounterValueTreeItem *stItem=dynamic_cast<FF_CounterValueTreeItem*>(grItem);
	  if (stItem) {
	    AE_COUNTER_VALUE *st=stItem->getObject();
            AE_CounterValue_Attach(st);
            AE_CounterValue_List2_PushBack(sl, st);
          }
        }
      }
      item=item->getBelow();
    }
    break;

  case COPYMODE_NONE:
  default:
    item=getFirstItem();
    while (item) {
      if (item->isSelected()) {
        FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
        if (grItem && (grItem->isDataItem())) {
          FF_CounterValueTreeItem *stItem=dynamic_cast<FF_CounterValueTreeItem*>(grItem);
          if (stItem)
            AE_CounterValue_List2_PushBack(sl, stItem->getObject());
        }
      }
      item=item->getBelow();
    }
    break;
  }
}



void FF_CounterValueTree::selectObject(AQDB_ID id, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_CounterValueTreeItem *stItem;

      stItem=dynamic_cast<FF_CounterValueTreeItem*>(grItem);
      if (stItem) {
        AE_COUNTER_VALUE *st;

        st=stItem->getObject();
        if (AE_CounterValue_GetId(st)==id) {
          stItem->setSelected(b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_CounterValueTree::selectObject(const AE_COUNTER_VALUE *fst, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_CounterValueTreeItem *stItem;

      stItem=dynamic_cast<FF_CounterValueTreeItem*>(grItem);
      if (stItem) {
        if (stItem->getObject()==fst) {
          stItem->setSelected(b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_CounterValueTree::makeObjectVisible(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_CounterValueTreeItem *stItem;

      stItem=dynamic_cast<FF_CounterValueTreeItem*>(grItem);
      if (stItem) {
        AE_COUNTER_VALUE *st;

        st=stItem->getObject();
        if (AE_CounterValue_GetId(st)==id) {
          makeItemVisible(stItem);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



GROUPTREE_SORT_FN_UP_DATE(FF_CounterValueTree, FF_CounterValueTreeItem, getObject, AE_COUNTER_VALUE, AE_CounterValue, Date)
GROUPTREE_SORT_FN_DOWN(FF_CounterValueTree, FF_CounterValueTreeItem, getObject, AE_COUNTER_VALUE, AE_CounterValue, Date)


GROUPTREE_SORT_FN_UP_VALUE(FF_CounterValueTree, FF_CounterValueTreeItem, getObject, AE_COUNTER_VALUE, AE_CounterValue, ValueX)
GROUPTREE_SORT_FN_DOWN(FF_CounterValueTree, FF_CounterValueTreeItem, getObject, AE_COUNTER_VALUE, AE_CounterValue, ValueX)

GROUPTREE_SORT_FN_UP_VALUE(FF_CounterValueTree, FF_CounterValueTreeItem, getObject, AE_COUNTER_VALUE, AE_CounterValue, ValueY)
GROUPTREE_SORT_FN_DOWN(FF_CounterValueTree, FF_CounterValueTreeItem, getObject, AE_COUNTER_VALUE, AE_CounterValue, ValueY)

GROUPTREE_SORT_FN_UP_VALUE(FF_CounterValueTree, FF_CounterValueTreeItem, getObject, AE_COUNTER_VALUE, AE_CounterValue, ValueZ)
GROUPTREE_SORT_FN_DOWN(FF_CounterValueTree, FF_CounterValueTreeItem, getObject, AE_COUNTER_VALUE, AE_CounterValue, ValueZ)


GROUPTREE_SORT_FN_UP_INT(FF_CounterValueTree, FF_CounterValueTreeItem, getObject, AE_COUNTER_VALUE, AE_CounterValue, DiffDays)
GROUPTREE_SORT_FN_DOWN(FF_CounterValueTree, FF_CounterValueTreeItem, getObject, AE_COUNTER_VALUE, AE_CounterValue, DiffDays)

GROUPTREE_SORT_FN_UP_VALUE(FF_CounterValueTree, FF_CounterValueTreeItem, getObject, AE_COUNTER_VALUE, AE_CounterValue, DiffX)
GROUPTREE_SORT_FN_DOWN(FF_CounterValueTree, FF_CounterValueTreeItem, getObject, AE_COUNTER_VALUE, AE_CounterValue, DiffX)

GROUPTREE_SORT_FN_UP_VALUE(FF_CounterValueTree, FF_CounterValueTreeItem, getObject, AE_COUNTER_VALUE, AE_CounterValue, DiffY)
GROUPTREE_SORT_FN_DOWN(FF_CounterValueTree, FF_CounterValueTreeItem, getObject, AE_COUNTER_VALUE, AE_CounterValue, DiffY)

GROUPTREE_SORT_FN_UP_VALUE(FF_CounterValueTree, FF_CounterValueTreeItem, getObject, AE_COUNTER_VALUE, AE_CounterValue, DiffZ)
GROUPTREE_SORT_FN_DOWN(FF_CounterValueTree, FF_CounterValueTreeItem, getObject, AE_COUNTER_VALUE, AE_CounterValue, DiffZ)







void FF_CounterValueTree::sortByColumn(int c, bool up) {
  if (c<m_columns) {
    switch(m_columnTypes[c]) {
    case COLUMN_DATE:
      setSortFunc(up?sortFunctionUpDate:sortFunctionDownDate);
      sortItems();
      break;

    case COLUMN_VALUEX:
      setSortFunc(up?sortFunctionUpValueX:sortFunctionDownValueX);
      sortItems();
      break;

    case COLUMN_VALUEY:
      setSortFunc(up?sortFunctionUpValueY:sortFunctionDownValueY);
      sortItems();
      break;

    case COLUMN_VALUEZ:
      setSortFunc(up?sortFunctionUpValueZ:sortFunctionDownValueZ);
      sortItems();
      break;

    case COLUMN_DIFFX:
      setSortFunc(up?sortFunctionUpDiffX:sortFunctionDownDiffX);
      sortItems();
      break;

    case COLUMN_DIFFY:
      setSortFunc(up?sortFunctionUpDiffY:sortFunctionDownDiffY);
      sortItems();
      break;

    case COLUMN_DIFFZ:
      setSortFunc(up?sortFunctionUpDiffZ:sortFunctionDownDiffZ);
      sortItems();
      break;

    default:
      DBG_ERROR(0, "Sort by column %d (%s) not implemented", c, up?"up":"down");
      break;
    }
  }
}



