/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2018 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ae_statementexport_p.h"

#include <aqbanking/types/transaction.h>

#include <gwenhywfar/text.h>
#include <gwenhywfar/debug.h>



int AE_StatementExport_StatementToTransaction(const AE_STATEMENT *st,
                                              AB_TRANSACTION *t)
{
  const char *s;
  const GWEN_DATE *dt;
  const AQDB_VALUE *v;
  uint32_t id;
  int i;
  const AE_RECURRENCE *rec;

  s=AE_Statement_GetLocalCountry(st);
  AB_Transaction_SetLocalCountry(t, s);

  s=AE_Statement_GetLocalBankCode(st);
  AB_Transaction_SetLocalBankCode(t, s);

  s=AE_Statement_GetLocalAccountNumber(st);
  AB_Transaction_SetLocalAccountNumber(t, s);

  s=AE_Statement_GetLocalIban(st);
  AB_Transaction_SetLocalIban(t, s);

  s=AE_Statement_GetLocalBic(st);
  AB_Transaction_SetLocalBic(t, s);

  s=AE_Statement_GetLocalName(st);
  AB_Transaction_SetLocalName(t, s);

  s=AE_Statement_GetRemoteCountry(st);
  AB_Transaction_SetRemoteCountry(t, s);

  s=AE_Statement_GetRemoteBankCode(st);
  AB_Transaction_SetRemoteBankCode(t, s);

  s=AE_Statement_GetRemoteAccountNumber(st);
  AB_Transaction_SetRemoteAccountNumber(t, s);

  s=AE_Statement_GetRemoteIban(st);
  AB_Transaction_SetRemoteIban(t, s);

  s=AE_Statement_GetRemoteBic(st);
  AB_Transaction_SetRemoteBic(t, s);

  s=AE_Statement_GetRemoteName(st);
  AB_Transaction_SetRemoteName(t, s);

  dt=AE_Statement_GetDate(st);
  AB_Transaction_SetDate(t, dt);

  dt=AE_Statement_GetValutaDate(st);
  AB_Transaction_SetValutaDate(t, dt);

  v=AE_Statement_GetValue(st);
  if (v) {
    GWEN_BUFFER *tbuf;
    AB_VALUE *nv;

    /* value */
    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
    AQDB_Value_toString(v, tbuf);
    nv=AB_Value_fromString(GWEN_Buffer_GetStart(tbuf));
    GWEN_Buffer_free(tbuf);
    /* currency */
    s=AE_Statement_GetCurrency(st);
    AB_Value_SetCurrency(nv, s);
    AB_Transaction_SetValue(t, nv);
    AB_Value_free(nv);
  }

  s=AE_Statement_GetFiId(st);
  AB_Transaction_SetFiId(t, s);

  i=AE_Statement_GetTextKey(st);
  AB_Transaction_SetTextKey(t, i);

  /*i=AE_Statement_GetTextKeyExt(st);*/
  /*AB_Transaction_SetTextKeyExt(t, i); no longer supported in AqBanking6 */

  s=AE_Statement_GetInfoText(st);
  AB_Transaction_SetTransactionText(t, s);

  s=AE_Statement_GetCustomerRef(st);
  AB_Transaction_SetCustomerReference(t, s);

  s=AE_Statement_GetBankRef(st);
  AB_Transaction_SetBankReference(t, s);

  s=AE_Statement_GetPurpose(st);
  AB_Transaction_SetPurpose(t, s);

  switch (AE_Statement_GetStatus(st)) {
  case AE_Statement_StatusUnknown:
    AB_Transaction_SetStatus(t, AB_Transaction_StatusUnknown);
    break;
  case AE_Statement_StatusNone:
    AB_Transaction_SetStatus(t, AB_Transaction_StatusNone);
    break;
  case AE_Statement_StatusEnqueued:
  case AE_Statement_StatusSending:
    AB_Transaction_SetStatus(t, AB_Transaction_StatusSending);
    break;
  case AE_Statement_StatusAccepted:
    AB_Transaction_SetStatus(t, AB_Transaction_StatusAccepted);
    break;
  case AE_Statement_StatusRejected:
    AB_Transaction_SetStatus(t, AB_Transaction_StatusRejected);
    break;
  case AE_Statement_StatusPending:
    AB_Transaction_SetStatus(t, AB_Transaction_StatusPending);
    break;
  case AE_Statement_StatusAutoReconciled:
    AB_Transaction_SetStatus(t, AB_Transaction_StatusAutoReconciled);
    break;
  case AE_Statement_StatusManuallyReconciled:
    AB_Transaction_SetStatus(t, AB_Transaction_StatusManuallyReconciled);
    break;
  case AE_Statement_StatusRevoked:
    AB_Transaction_SetStatus(t, AB_Transaction_StatusRevoked);
    break;
  case AE_Statement_StatusAborted:
    AB_Transaction_SetStatus(t, AB_Transaction_StatusAborted);
    break;
  case AE_Statement_StatusSent:
    AB_Transaction_SetStatus(t, AB_Transaction_StatusSent);
    break;
  }

  id=AE_Statement_GetGroupId(st);
  AB_Transaction_SetGroupId(t, id);

  id=AE_Statement_GetId(st);
  AB_Transaction_SetIdForApplication(t, id);

  s=AE_Statement_GetCategory(st);
  AB_Transaction_SetCategory(t, s);

  rec=AE_Statement_GetRecurrence(st);
  if (rec) {
    int i;

    DBG_ERROR(0, "Recurrence found");

    i=AE_Recurrence_GetPeriod(rec);
    switch (i) {
    case AE_Recurrence_PeriodWeeks:
      AB_Transaction_SetPeriod(t, AB_Transaction_PeriodWeekly);
      switch (AE_Recurrence_GetOn(rec)) {
      case 1:
        AB_Transaction_SetExecutionDay(t, 1);
        break;
      case 2:
        AB_Transaction_SetExecutionDay(t, 2);
        break;
      case 3:
        AB_Transaction_SetExecutionDay(t, 3);
        break;
      case 4:
        AB_Transaction_SetExecutionDay(t, 4);
        break;
      case 5:
        AB_Transaction_SetExecutionDay(t, 5);
        break;
      case 6:
        AB_Transaction_SetExecutionDay(t, 6);
        break;
      case 7:
        AB_Transaction_SetExecutionDay(t, 7);
        break;
      default:
        break;
      }

      break;

    case AE_Recurrence_PeriodMonths:
      AB_Transaction_SetPeriod(t, AB_Transaction_PeriodMonthly);
      AB_Transaction_SetExecutionDay(t, AE_Recurrence_GetOn(rec));
      break;

    case AE_Recurrence_PeriodDays:
    case AE_Recurrence_PeriodYears:
    default:
      DBG_WARN(AE_LOGDOMAIN, "Period %d not supported", i);
      break;
    }
    AB_Transaction_SetCycle(t, AE_Recurrence_GetEvery(rec));

    dt=AE_Recurrence_GetBeginDate(rec);
    AB_Transaction_SetFirstDate(t, dt);

    dt=AE_Recurrence_GetEndDate(rec);
    AB_Transaction_SetLastDate(t, dt);

    dt=AE_Recurrence_GetNextExecutionDate(rec);
    AB_Transaction_SetNextDate(t, dt);
  }

  s=AE_Statement_GetEndToEndId(st);
  AB_Transaction_SetEndToEndReference(t, s);


  if (0) {
    GWEN_DB_NODE *dbD;

    dbD=GWEN_DB_Group_new("debug");
    AB_Transaction_toDb(t, dbD);
    GWEN_DB_Dump(dbD, 2);
    GWEN_DB_Group_free(dbD);
  }

  return 0;
}




void GWENHYWFAR_CB *AE_StatementExport_ExportStatement(AE_BOOK *bk,
                                                       AE_BOOK_TABLE_TYPE tt,
                                                       AQDB_OBJECT *o,
                                                       void *data)
{
  AB_IMEXPORTER_CONTEXT *iec;
  AE_STATEMENT *st=NULL;
  int rv;

  iec=(AB_IMEXPORTER_CONTEXT *) data;
  rv=AE_Statement_fromObject(o, &st);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
  }
  else {
    AB_TRANSACTION *t;

    t=AB_Transaction_new();
    rv=AE_StatementExport_StatementToTransaction(st, t);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    }
    else {
      AB_ImExporterContext_AddTransaction(iec, t);
    }
  }

  return NULL;
}



int AE_StatementExport_ExportContext(AE_BOOK *b,
                                     AB_IMEXPORTER_CONTEXT *iec,
                                     AE_BOOK_TABLE_TYPE tt,
                                     const char *queryString)
{
  int rv;
  void *dummy;

  if (tt!=AE_Book_TableType_BankTransfer &&
      tt!=AE_Book_TableType_BankStatement) {
    DBG_ERROR(AE_LOGDOMAIN, "Invalid table type %d", (int) tt);
    return GWEN_ERROR_INVALID;
  }

  rv=AE_Book_ForEveryQueryObject(b,
                                 tt,
                                 0,
                                 queryString,
                                 AE_StatementExport_ExportStatement,
                                 (void *) iec,
                                 &dummy);
  if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
    DBG_ERROR(0, "No entries found");
  }

  return 0;
}






