/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2013 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "af_report2_p.h"

#include <gwenhywfar/debug.h>

#include <aqfinance/graphics/af_graphics_cairo.h>

#include <aqfinance/graphs/af_graph_dataset.h>
#include <aqfinance/graphs/af_graph.h>

#include <aqfinance/report2/af_report2_settings.h>
#include <aqfinance/engine/book/ae_counter_value.h>



GWEN_LIST_FUNCTIONS(AF_REPORT2, AF_Report2)
GWEN_LIST2_FUNCTIONS(AF_REPORT2, AF_Report2)


#define AF_REPORT2_FLOATING_AVERAGE_NUM 20



AF_REPORT2 *AF_Report2_new()
{
  AF_REPORT2 *rp;

  GWEN_NEW_OBJECT(AF_REPORT2, rp);
  assert(rp);
  rp->_refCount=1;
  GWEN_LIST_INIT(AF_REPORT2, rp);

  rp->categoryTree=AF_Report2_Category_Tree_new();

  return rp;
}



void AF_Report2_free(AF_REPORT2 *rp)
{
  if (rp) {
    assert(rp->_refCount>0);
    if (rp->_refCount==1) {
      GWEN_LIST_FINI(AF_REPORT2, rp)
      AF_Report2_Category_Tree_free(rp->categoryTree);
      rp->_refCount=0;
      GWEN_FREE_OBJECT(rp);
    }
    else {
      rp->_refCount--;
    }
  }
}



AF_REPORT2_CATEGORY_TREE *AF_Report2_GetCategoryTree(const AF_REPORT2 *rp)
{
  assert(rp);
  assert(rp->_refCount);
  return rp->categoryTree;
}



int AF_Report2_AddCategory(AF_REPORT2 *rp, AF_REPORT2_CATEGORY *cat, AQDB_ID parentId)
{
  if (rp->categoryTree==NULL)
    rp->categoryTree=AF_Report2_Category_Tree_new();

  if (parentId) {
    AF_REPORT2_CATEGORY *parentCat;

    parentCat=AF_Report2_Category_Tree_GetById(rp->categoryTree, parentId);
    if (parentCat==NULL) {
      AF_Report2_Category_Tree_AddChild(parentCat, cat);
      return 0;
    }
    DBG_ERROR(0, "Parent category not found, adding to root");
  }
  AF_Report2_Category_Tree_Add(rp->categoryTree, cat);
  return 0;
}



int AF_Report2_AddDataPair(AF_REPORT2 *rp, AF_REPORT2_DATAPAIR *dp)
{
  AF_REPORT2_CATEGORY *cat;
  AQDB_ID tmpId;

  assert(rp);
  assert(dp);

  tmpId=AF_Report2_DataPair_GetCategoryId(dp);
  if (rp->categoryTree==NULL)
    rp->categoryTree=AF_Report2_Category_Tree_new();
  cat=AF_Report2_Category_Tree_GetById(rp->categoryTree, tmpId);
  if (cat==NULL) {
    DBG_INFO(0, "Ignoring datapair for non-existing category");
    return GWEN_ERROR_NOT_FOUND;
  }

  return AF_Report2_Category_AddDataPair(cat, dp);
}



int AF_Report2_CollapseSubCategories(AF_REPORT2 *rp, AF_REPORT2_CATEGORY *cat)
{
  AF_REPORT2_CATEGORY *cc;

  while ((cc=AF_Report2_Category_Tree_GetFirstChild(cat))) {
    AF_REPORT2_YEARINFO_LIST *yil;

    /* recurse into sub categories */
    AF_Report2_CollapseSubCategories(rp, cc);

    /* move entries */
    yil=AF_Report2_Category_GetYearInfoList(cc);
    if (yil) {
      AF_REPORT2_YEARINFO *yi;

      yi=AF_Report2_YearInfo_List_First(yil);
      while (yi) {
        AF_REPORT2_MONTHINFO_LIST *mil;

        mil=AF_Report2_YearInfo_GetMonthInfoList(yi);
        if (mil) {
          AF_REPORT2_MONTHINFO *mi;

          mi=AF_Report2_MonthInfo_List_First(mil);
          while (mi) {
            AF_REPORT2_DAYINFO_LIST *dil;

            dil=AF_Report2_MonthInfo_GetDayInfoList(mi);
            if (dil) {
              AF_REPORT2_DAYINFO *di;

              di=AF_Report2_DayInfo_List_First(dil);
              while (di) {
                AF_REPORT2_DATAPAIR_LIST *dpl;

                dpl=AF_Report2_DayInfo_GetDataPairList(di);
                if (dpl) {
                  AF_REPORT2_DATAPAIR *dp;

                  while ((dp=AF_Report2_DataPair_List_First(dpl))) {
                    AF_Report2_DataPair_List_Del(dp);
                    AF_Report2_Category_AddDataPair(cat, dp);
                  }
                }
                di=AF_Report2_DayInfo_List_Next(di);
              } /* while di */
            }
            mi=AF_Report2_MonthInfo_List_Next(mi);
          } /* while mi */
        }

        yi=AF_Report2_YearInfo_List_Next(yi);
      }
    }

    AF_Report2_Category_Tree_Del(cc);
  }

  return 0;
}



void AF_Report2_CollapseAllCategories(AF_REPORT2 *rp)
{
  AF_REPORT2_CATEGORY *cat;
  AF_REPORT2_CATEGORY *cc;

  cat=AF_Report2_Category_new();

  cc=AF_Report2_Category_Tree_GetFirst(rp->categoryTree);
  while (cc) {
    AF_REPORT2_YEARINFO_LIST *yil;

    DBG_ERROR(0, "Handling category");
    AF_Report2_CollapseSubCategories(rp, cc);

    /* move entries of all children to new category */
    yil=AF_Report2_Category_GetYearInfoList(cc);
    if (yil) {
      AF_REPORT2_YEARINFO *yi;

      yi=AF_Report2_YearInfo_List_First(yil);
      while (yi) {
        AF_REPORT2_MONTHINFO_LIST *mil;

        DBG_ERROR(0, "  Handling year info");
        mil=AF_Report2_YearInfo_GetMonthInfoList(yi);
        if (mil) {
          AF_REPORT2_MONTHINFO *mi;

          mi=AF_Report2_MonthInfo_List_First(mil);
          while (mi) {
            AF_REPORT2_DAYINFO_LIST *dil;

            DBG_ERROR(0, "    Handling month info");
            dil=AF_Report2_MonthInfo_GetDayInfoList(mi);
            if (dil) {
              AF_REPORT2_DAYINFO *di;

              di=AF_Report2_DayInfo_List_First(dil);
              while (di) {
                AF_REPORT2_DATAPAIR_LIST *dpl;

                DBG_ERROR(0, "      Handling day info");
                dpl=AF_Report2_DayInfo_GetDataPairList(di);
                if (dpl) {
                  AF_REPORT2_DATAPAIR *dp;

                  while ((dp=AF_Report2_DataPair_List_First(dpl))) {
                    AF_Report2_DataPair_List_Del(dp);
                    AF_Report2_Category_AddDataPair(cat, dp);
                  }
                }
                di=AF_Report2_DayInfo_List_Next(di);
              } /* while di */
            }
            mi=AF_Report2_MonthInfo_List_Next(mi);
          } /* while mi */
        }

        yi=AF_Report2_YearInfo_List_Next(yi);
      }
    }

    cc=AF_Report2_Category_Tree_GetNext(cc);
  }

  /* remove all categories and add the new category instead */
  AF_Report2_Category_Tree_Clear(rp->categoryTree);
  AF_Report2_Category_Tree_Add(rp->categoryTree, cat);
}



void AF_Report2_RemoveEmptySubCategories(AF_REPORT2 *rp, AF_REPORT2_CATEGORY *cat)
{
  AF_REPORT2_CATEGORY *cc;

  cc=AF_Report2_Category_Tree_GetFirstChild(cat);
  while (cc) {
    AF_REPORT2_CATEGORY *ccNext;
    AF_REPORT2_YEARINFO_LIST *yil;
    int keep=0;

    ccNext=AF_Report2_Category_Tree_GetNext(cc);

    /* recurse into sub categories */
    AF_Report2_RemoveEmptySubCategories(rp, cc);

    /* move entries */
    yil=AF_Report2_Category_GetYearInfoList(cat);
    if (yil && AF_Report2_YearInfo_List_First(yil))
      keep=1;

    if (AF_Report2_Category_Tree_GetFirstChild(cc))
      keep=1;

    if (!keep)
      AF_Report2_Category_Tree_Del(cc);

    cc=ccNext;
  }
}



static void AF_Report2__CollectDayDataForCategory(AF_REPORT2 *rp, AF_REPORT2_CATEGORY *cat, AF_GRAPH_DATASET *ds,
                                                  int level, uint32_t flags)
{
  AF_REPORT2_YEARINFO_LIST *yil;
  AF_REPORT2_CATEGORY *cc;

  yil=AF_Report2_Category_GetYearInfoList(cat);
  if (yil) {
    AF_REPORT2_YEARINFO *yi;

    yi=AF_Report2_YearInfo_List_First(yil);
    while (yi) {
      AF_REPORT2_MONTHINFO_LIST *mil;

      mil=AF_Report2_YearInfo_GetMonthInfoList(yi);
      if (mil) {
        AF_REPORT2_MONTHINFO *mi;

        mi=AF_Report2_MonthInfo_List_First(mil);
        while (mi) {
          AF_REPORT2_DAYINFO_LIST *dil;

          dil=AF_Report2_MonthInfo_GetDayInfoList(mi);
          if (dil) {
            AF_REPORT2_DAYINFO *di;

            di=AF_Report2_DayInfo_List_First(dil);
            while (di) {
              AF_GRAPH_DATAPAIR *dp;
              const GWEN_DATE *dt;
              double v;

              dp=AF_Graph_DataPair_new();
              AF_Graph_DataPair_SetLevel(dp, level);

              DBG_ERROR(0, "Adding value (total=%f, pos=%f, neg=%f)",
                        AF_Report2_DayInfo_GetTotalSum(di),
                        AF_Report2_DayInfo_GetPositiveSum(di),
                        AF_Report2_DayInfo_GetNegativeSum(di));

              dt=AF_Report2_DayInfo_GetDate(di);
              if (dt)
                AF_Graph_DataPair_SetValueX(dp, GWEN_Date_GetJulian(dt));
              switch (flags & AF_REPORT2_FLAGS_SUM_MASK) {

              case AF_REPORT2_FLAGS_SUM_POSITIVE:
                v=AF_Report2_DayInfo_GetPositiveSum(di);
                break;

              case AF_REPORT2_FLAGS_SUM_NEGATIVE:
                v=AF_Report2_DayInfo_GetNegativeSum(di);
                break;

              default:
              case AF_REPORT2_FLAGS_SUM_TOTAL:
                v=AF_Report2_DayInfo_GetTotalSum(di);
                break;
              }

              if (flags & AF_REPORT2_FLAGS_REVERSE_SIGN)
                v=-v;
              AF_Graph_DataPair_SetValueY(dp, v);

              AF_Graph_DataSet_AddDataPair(ds, dp);

              di=AF_Report2_DayInfo_List_Next(di);
            }
          }

          mi=AF_Report2_MonthInfo_List_Next(mi);
        }
      }

      yi=AF_Report2_YearInfo_List_Next(yi);
    }

  }

  cc=AF_Report2_Category_Tree_GetFirstChild(cat);
  while (cc) {
    AF_Report2__CollectDayDataForCategory(rp, cc, ds, level, flags);
    cc=AF_Report2_Category_Tree_GetNext(cc);
  }
}



static void AF_Report2__CollectMonthDataForCategory(AF_REPORT2 *rp, AF_REPORT2_CATEGORY *cat, AF_GRAPH_DATASET *ds,
                                                    int level, uint32_t flags)
{
  AF_REPORT2_YEARINFO_LIST *yil;
  AF_REPORT2_CATEGORY *cc;

  yil=AF_Report2_Category_GetYearInfoList(cat);
  if (yil) {
    AF_REPORT2_YEARINFO *yi;

    yi=AF_Report2_YearInfo_List_First(yil);
    while (yi) {
      AF_REPORT2_MONTHINFO_LIST *mil;

      mil=AF_Report2_YearInfo_GetMonthInfoList(yi);
      if (mil) {
        AF_REPORT2_MONTHINFO *mi;

        mi=AF_Report2_MonthInfo_List_First(mil);
        while (mi) {
          AF_GRAPH_DATAPAIR *dp;
          GWEN_DATE *dt;
          AQDB_ID id;
          double v;

          id=AF_Report2_MonthInfo_GetId(mi);
          dt=GWEN_Date_fromGregorian((id>>16) & 0xffff, (id>>8) & 0xff, 1);

          dp=AF_Graph_DataPair_new();
          AF_Graph_DataPair_SetLevel(dp, level);
          AF_Graph_DataPair_SetValueX(dp, GWEN_Date_GetJulian(dt));
          GWEN_Date_free(dt);

          switch (flags & AF_REPORT2_FLAGS_SUM_MASK) {

          case AF_REPORT2_FLAGS_SUM_POSITIVE:
            v=AF_Report2_MonthInfo_GetPositiveSum(mi);
            break;

          case AF_REPORT2_FLAGS_SUM_NEGATIVE:
            v=AF_Report2_MonthInfo_GetNegativeSum(mi);
            break;

          default:
          case AF_REPORT2_FLAGS_SUM_TOTAL:
            v=AF_Report2_MonthInfo_GetTotalSum(mi);
            break;
          }

          if (flags & AF_REPORT2_FLAGS_REVERSE_SIGN)
            v=-v;
          AF_Graph_DataPair_SetValueY(dp, v);
          AF_Graph_DataSet_AddDataPair(ds, dp);

          mi=AF_Report2_MonthInfo_List_Next(mi);
        }
      }

      yi=AF_Report2_YearInfo_List_Next(yi);
    }

  }

  cc=AF_Report2_Category_Tree_GetFirstChild(cat);
  while (cc) {
    AF_Report2__CollectMonthDataForCategory(rp, cc, ds, level, flags);
    cc=AF_Report2_Category_Tree_GetNext(cc);
  }
}



/*
 * Creates a data pair for the given category.
 * - x-value will be the amount stored for the category
 * - y-value will be the category number
 * - label will be the name of the category
 */
static void AF_Report2__CollectCategoryDataForCategory(AF_REPORT2 *rp, AF_REPORT2_CATEGORY *cat,
                                                       AF_GRAPH_DATASET *ds,
                                                       int level, int num, uint32_t flags)
{
  AF_GRAPH_DATAPAIR *dp;
  double v;

  dp=AF_Graph_DataPair_new();
  AF_Graph_DataPair_SetLabel(dp, AF_Report2_Category_GetName(cat));
  AF_Graph_DataPair_SetLevel(dp, level);
  AF_Graph_DataPair_SetValueX(dp, num);

  switch (flags & AF_REPORT2_FLAGS_SUM_MASK) {

  case AF_REPORT2_FLAGS_SUM_POSITIVE:
    v=AF_Report2_Category_GetPositiveSum(cat);
    break;

  case AF_REPORT2_FLAGS_SUM_NEGATIVE:
    v=AF_Report2_Category_GetNegativeSum(cat);
    break;

  default:
  case AF_REPORT2_FLAGS_SUM_TOTAL:
    v=AF_Report2_Category_GetTotalSum(cat);
    break;
  }

  if (flags & AF_REPORT2_FLAGS_REVERSE_SIGN)
    v=-v;
  DBG_ERROR(0, "category [%s]: %f", AF_Graph_DataPair_GetLabel(dp), v);
  AF_Graph_DataPair_SetValueY(dp, v);
  AF_Graph_DataSet_AddDataPair(ds, dp);
}



void AF_Report2_AddCategoryData2DataSet(AF_REPORT2 *rp, AF_GRAPH_DATASET *ds, uint32_t flags,
                                        const char *label,
                                        int level, int graphType, int colour, int lineWidth, int dashType)
{
  AF_REPORT2_CATEGORY *cat;
  int idx;
  AF_GRAPH_DATAPAIR_LIST *dpl;

  AF_Graph_DataSet_AddSubGraph(ds, label, 1, graphType, AF_Report2_DataMode_Direct, colour, lineWidth, dashType);

  idx=1;
  cat=AF_Report2_Category_Tree_GetFirst(rp->categoryTree);
  while (cat) {
    AF_Report2__CollectCategoryDataForCategory(rp, cat, ds, level, idx++, flags);
    cat=AF_Report2_Category_Tree_GetNext(cat);
  }

  /* sort (descending, i.e. lowest amount first) */
  AF_Graph_DataSet_SortDataPairsByY(ds, 0);

  AF_Graph_DataSet_KeepFirstNValues(ds, 10);

  /* assign y-values to every entry according to their order
   * this means the category with the highest amount receives the highest Y value
   */
  dpl=AF_Graph_DataSet_GetDataPairList(ds);
  if (dpl) {
    AF_GRAPH_DATAPAIR *dp;

    idx=2;
    dp=AF_Graph_DataPair_List_First(dpl);
    while (dp) {
      AF_Graph_DataPair_SetValueX(dp, idx++);
      dp=AF_Graph_DataPair_List_Next(dp);
    }
  }
}




void AF_Report2_AddDayData2DataSet(AF_REPORT2 *rp, AF_GRAPH_DATASET *ds, uint32_t flags,
                                   const char *label,
                                   int level, int graphType, int dataMode, int colour, int lineWidth, int dashType)
{
  AF_REPORT2_CATEGORY *cat;

  AF_Graph_DataSet_AddSubGraph(ds, label, level, graphType, dataMode, colour, lineWidth, dashType);

  cat=AF_Report2_Category_Tree_GetFirst(rp->categoryTree);
  while (cat) {
    AF_Report2__CollectDayDataForCategory(rp, cat, ds, level, flags);
    cat=AF_Report2_Category_Tree_GetNext(cat);
  }

  AF_Graph_DataSet_SortDataPairsByX(ds, 1);
  if (flags & AF_REPORT2_FLAGS_ACCUMULATE)
    AF_Graph_DataSet_AccumulateDataPairs(ds, level);

  switch (dataMode) {
  case AF_Report2_DataMode_Direct:
    break;

  case AF_Report2_DataMode_Average:
    AF_Graph_DataSet_AverageDataPairs(ds, level);
    break;
  case AF_Report2_DataMode_FloatingAverage:
    AF_Graph_DataSet_FloatingAverageDataPairs(ds, level, AF_REPORT2_FLOATING_AVERAGE_NUM);
    break;
  }
}



void AF_Report2_AddMonthData2DataSet(AF_REPORT2 *rp, AF_GRAPH_DATASET *ds, uint32_t flags,
                                     const char *label,
                                     int level, int graphType, int dataMode, int colour, int lineWidth, int dashType)
{
  AF_REPORT2_CATEGORY *cat;

  AF_Graph_DataSet_AddSubGraph(ds, label, level, graphType, dataMode, colour, lineWidth, dashType);

  cat=AF_Report2_Category_Tree_GetFirst(rp->categoryTree);
  while (cat) {
    AF_Report2__CollectMonthDataForCategory(rp, cat, ds, level, flags);
    cat=AF_Report2_Category_Tree_GetNext(cat);
  }

  AF_Graph_DataSet_SortDataPairsByX(ds, 1);
  if (flags & AF_REPORT2_FLAGS_ACCUMULATE)
    AF_Graph_DataSet_AccumulateDataPairs(ds, level);

  switch (dataMode) {
  case AF_Report2_DataMode_Direct:
    break;

  case AF_Report2_DataMode_Average:
    AF_Graph_DataSet_AverageDataPairs(ds, level);
    break;
  case AF_Report2_DataMode_FloatingAverage:
    AF_Graph_DataSet_FloatingAverageDataPairs(ds, level, AF_REPORT2_FLOATING_AVERAGE_NUM);
    break;
  }
}



int AF_Report2_CreateReport(AE_BOOK *b, const AF_REPORT2_SETTINGS *settings, const char *fname)
{
  AF_GRAPH_DATASET *ds;

  /* prepare data set */
  ds=AF_Graph_DataSet_new();
  AF_Graph_DataSet_SetTitle(ds, AF_Report2_Settings_GetTitle(settings));
  AF_Graph_DataSet_SetSubTitle(ds, AF_Report2_Settings_GetSubTitle(settings));

  AF_Graph_DataSet_SetDataTypeX(ds, AF_Graph_DataSet_DataType_Date);
  AF_Graph_DataSet_SetLabelX(ds, I18N("Date"));

  AF_Graph_DataSet_SetDataTypeY(ds, AF_Graph_DataSet_DataType_Double);
  AF_Graph_DataSet_SetLabelY(ds, AF_Report2_Settings_GetLabelY(settings));
  AF_Graph_DataSet_SetPrecisionY(ds, AF_Report2_Settings_GetPrecisionY(settings));

  /* sample data into data set */
  {
    const AF_REPORT2_SETTINGS_GRAPH_LIST *sgList;
    const char *commodity;
    int rv;
    GWEN_DATE *firstDate=NULL;
    GWEN_DATE *lastDate=NULL;
    GWEN_DATE *today=GWEN_Date_CurrentDate();

    commodity=AF_Report2_Settings_GetCommodity(settings);
    if (!(commodity && *commodity))
      commodity="EUR";

    /* get dates */
    switch (AF_Report2_Settings_GetDateSpanType(settings)) {
    case AF_Report2_DateSpanType_None:
      break;

    case AF_Report2_DateSpanType_UserDefined: {
      const GWEN_DATE *dt;

      dt=AF_Report2_Settings_GetFirstDate(settings);
      if (dt)
        firstDate=GWEN_Date_dup(dt);
      dt=AF_Report2_Settings_GetLastDate(settings);
      if (dt)
        lastDate=GWEN_Date_dup(dt);
      break;
    }

    case AF_Report2_DateSpanType_ThisMonth:
      firstDate=GWEN_Date_GetThisMonthStart(today);
      lastDate=GWEN_Date_GetThisMonthEnd(today);
      break;

    case AF_Report2_DateSpanType_ThisQuarterYear:
      firstDate=GWEN_Date_GetThisQuarterYearStart(today);
      lastDate=GWEN_Date_GetThisQuarterYearEnd(today);
      break;

    case AF_Report2_DateSpanType_ThisHalfYear:
      firstDate=GWEN_Date_GetThisHalfYearStart(today);
      lastDate=GWEN_Date_GetThisHalfYearEnd(today);
      break;

    case AF_Report2_DateSpanType_ThisYear:
      firstDate=GWEN_Date_GetThisYearStart(today);
      lastDate=GWEN_Date_GetThisYearEnd(today);
      break;

    case AF_Report2_DateSpanType_LastMonth:
      firstDate=GWEN_Date_GetLastMonthStart(today);
      lastDate=GWEN_Date_GetLastMonthEnd(today);
      break;

    case AF_Report2_DateSpanType_LastQuarterYear:
      firstDate=GWEN_Date_GetLastQuarterYearStart(today);
      lastDate=GWEN_Date_GetLastQuarterYearEnd(today);
      break;

    case AF_Report2_DateSpanType_LastHalfYear:
      firstDate=GWEN_Date_GetLastHalfYearStart(today);
      lastDate=GWEN_Date_GetLastHalfYearEnd(today);
      break;

    case AF_Report2_DateSpanType_LastYear:
      firstDate=GWEN_Date_GetLastYearStart(today);
      lastDate=GWEN_Date_GetLastYearEnd(today);
      break;
    }


    sgList=AF_Report2_Settings_GetGraphList(settings);
    if (sgList) {
      const AF_REPORT2_SETTINGS_GRAPH *sg;
      int level=1;
      AE_ACCOUNT_TREE *at;

      at=AE_Account_Tree_new();
      rv=AE_Book_ReadAccountTree(b, AE_Book_TableType_Account, at);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        AE_Account_Tree_free(at);
        GWEN_Date_free(today);
        GWEN_Date_free(lastDate);
        GWEN_Date_free(firstDate);
        return rv;
      }

      sg=AF_Report2_SettingsGraph_List_First(sgList);
      while (sg) {
        if (AF_Report2_SettingsGraph_GetFlags(sg) & AF_REPORT2_FLAGS_ENABLED) {
          AF_REPORT2 *rp;
          AF_REPORT2_CATEGORY_TREE *catTree;
          GWEN_STRINGLIST2 *sl;

          rp=AF_Report2_new();
          catTree=AF_Report2_GetCategoryTree(rp);
          rv=AF_Book_Report2_AccountTree2CategoryTree(b, catTree);
          if (rv<0) {
            DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
            AE_Account_Tree_free(at);
            GWEN_Date_free(today);
            GWEN_Date_free(lastDate);
            GWEN_Date_free(firstDate);
            return rv;
          }

          AE_Account_Tree_SubRuntimeFlags(at, AE_ACCOUNT_RTFLAGS_SELECTED);
          sl=AF_Report2_SettingsGraph_GetCategoryNameList(sg);
          if (sl) {
            GWEN_STRINGLIST2_ITERATOR *sli;
            AE_SPLIT_LIST *spList;

            /* set matching accounts SELECTED in account tree */
            sli=GWEN_StringList2_First(sl);
            if (sli) {
              const char *s;

              s=GWEN_StringList2Iterator_Data(sli);
              while (s) {
                AE_Account_Tree_SelectAccountAndChildrenByName(at, s, 1);
                s=GWEN_StringList2Iterator_Next(sli);
              }
              GWEN_StringList2Iterator_free(sli);
            }

            spList=AE_Split_List_new();
            rv=AE_Book_ReadSplitListForSelectedAccounts(b, at, firstDate, lastDate, commodity, spList);
            if (rv<0) {
              DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
              AE_Split_List_free(spList);
              AE_Account_Tree_free(at);
              AF_Report2_free(rp);
              GWEN_Date_free(today);
              GWEN_Date_free(lastDate);
              GWEN_Date_free(firstDate);
              return rv;
            }

            if (AE_Split_List_GetCount(spList)==0) {
              DBG_ERROR(AE_LOGDOMAIN, "No data for graph, aborting");
              AE_Split_List_free(spList);
              AE_Account_Tree_free(at);
              AF_Report2_free(rp);
              GWEN_Date_free(today);
              GWEN_Date_free(lastDate);
              GWEN_Date_free(firstDate);
              return GWEN_ERROR_NO_DATA;
            }


            AF_Book_Report2_SplitList2Report(b, rp, spList);
            AE_Split_List_free(spList);
          }

          AF_Report2_CollapseAllCategories(rp);

          switch (AF_Report2_Settings_GetDateUnits(settings)) {
          case AF_Report2_DateUnits_Days:
            AF_Report2_AddDayData2DataSet(rp, ds,
                                          AF_Report2_SettingsGraph_GetFlags(sg),
                                          NULL,
                                          level,
                                          AF_Report2_SettingsGraph_GetGraphType(sg),
                                          AF_Report2_SettingsGraph_GetDataMode(sg),
                                          AF_Report2_SettingsGraph_GetColour(sg),
                                          AF_Report2_SettingsGraph_GetLineWidth(sg),
                                          AF_Report2_SettingsGraph_GetDashType(sg));
            break;

          default:
          case AF_Report2_DateUnits_Months:
            AF_Report2_AddMonthData2DataSet(rp, ds,
                                            AF_Report2_SettingsGraph_GetFlags(sg),
                                            NULL,
                                            level,
                                            AF_Report2_SettingsGraph_GetGraphType(sg),
                                            AF_Report2_SettingsGraph_GetDataMode(sg),
                                            AF_Report2_SettingsGraph_GetColour(sg),
                                            AF_Report2_SettingsGraph_GetLineWidth(sg),
                                            AF_Report2_SettingsGraph_GetDashType(sg));
            break;

#if 0
          case AF_Report2_DateUnits_Years:
            AF_Report2_AddYearData2DataSet(rp, ds,
                                           AF_Report2_SettingsGraph_GetFlags(sg),
                                           NULL,
                                           level,
                                           AF_Report2_SettingsGraph_GetGraphType(sg),
                                           AF_Report2_SettingsGraph_GetDataMode(sg),
                                           AF_Report2_SettingsGraph_GetColour(sg),
                                           AF_Report2_SettingsGraph_GetLineWidth(sg),
                                           AF_Report2_SettingsGraph_GetDashType(sg));
            break;
#endif
          }
          AF_Report2_free(rp);

          level++;
        }
        sg=AF_Report2_SettingsGraph_List_Next(sg);
      }

      AE_Account_Tree_free(at);
    }

    GWEN_Date_free(today);
    GWEN_Date_free(lastDate);
    GWEN_Date_free(firstDate);
  }

  /* prepare drawing of data set */
  AF_Graph_DataSet_AdjustXRange(ds);
  AF_Graph_DataSet_AdjustYRange(ds);


  /* add ticks */
#if 0
  AF_Graph_DataSet_AddTicks(ds);
#else
  {
    AF_GRAPH_TICK_LIST *tl;
    int level=1;
    int dist;

    tl=AF_Graph_DataSet_GetYTickList(ds);
    if (tl==NULL) {
      tl=AF_Graph_Tick_List_new();
      AF_Graph_DataSet_SetYTickList(ds, tl);
    }
    AF_Graph_Tick_AddTicks(tl,
                           AF_Graph_DataSet_GetDataTypeY(ds),
                           AF_Graph_DataSet_GetMinValueY(ds),
                           AF_Graph_DataSet_GetMaxValueY(ds),
                           AF_Graph_DataSet_GetPrecisionY(ds));


    tl=AF_Graph_DataSet_GetXTickList(ds);
    if (tl==NULL) {
      tl=AF_Graph_Tick_List_new();
      AF_Graph_DataSet_SetXTickList(ds, tl);
    }

    dist=AF_Graph_DataSet_GetMaxValueX(ds)-AF_Graph_DataSet_GetMinValueX(ds);

    /* add X ticks */
    switch (AF_Report2_Settings_GetDateUnits(settings)) {
    case AF_Report2_DateUnits_Days:
      if (dist>365)
        AF_Graph_Tick_AddTicks_DateYears(tl, level++, AF_Graph_DataSet_GetMinValueX(ds), AF_Graph_DataSet_GetMaxValueX(ds));
      if (dist>30)
        AF_Graph_Tick_AddTicks_DateMonths(tl, level++, AF_Graph_DataSet_GetMinValueX(ds), AF_Graph_DataSet_GetMaxValueX(ds));
      if (dist>7)
        AF_Graph_Tick_AddTicks_DateWeeks(tl, level++, AF_Graph_DataSet_GetMinValueX(ds), AF_Graph_DataSet_GetMaxValueX(ds));
      if (dist>1)
        AF_Graph_Tick_AddTicks_DateDays(tl, level++, AF_Graph_DataSet_GetMinValueX(ds), AF_Graph_DataSet_GetMaxValueX(ds));
      break;

    default:
    case AF_Report2_DateUnits_Months:
      if (dist>365)
        AF_Graph_Tick_AddTicks_DateYears(tl, level++, AF_Graph_DataSet_GetMinValueX(ds), AF_Graph_DataSet_GetMaxValueX(ds));
      if (dist>30)
        AF_Graph_Tick_AddTicks_DateMonths(tl, level++, AF_Graph_DataSet_GetMinValueX(ds), AF_Graph_DataSet_GetMaxValueX(ds));
      break;

    case AF_Report2_DateUnits_Years:
      if (dist>365)
        AF_Graph_Tick_AddTicks_DateYears(tl, level++, AF_Graph_DataSet_GetMinValueX(ds), AF_Graph_DataSet_GetMaxValueX(ds));
      break;
    }
  }
#endif


  /* actually draw */
  {
    AF_GRAPHICS *g;
    AF_GRAPH *gr;
    const AF_GRAPH_SETTINGS *gs;
    int rv;

    g=AF_GraphicsCairo_Png_new(fname, AF_Report2_Settings_GetWidth(settings), AF_Report2_Settings_GetHeight(settings));
    AF_Graphics_Attach(g);
    gs=AF_Report2_Settings_GetGraphSettings(settings);
    if (gs==NULL) {
      DBG_ERROR(AE_LOGDOMAIN, "No graph settings");
      AF_Graphics_free(g);
      return GWEN_ERROR_INVALID;
    }
    gr=AF_Graph_new(ds, g, AF_Report2_Settings_GetWidth(settings), AF_Report2_Settings_GetHeight(settings));
    rv=AF_Graph_Init(gr, gs);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AF_Graphics_free(g);
      AF_Graph_free(gr);
      return rv;
    }

    rv=AF_Graph_Draw(gr);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AF_Graphics_free(g);
      AF_Graph_free(gr);
      return rv;
    }

    AF_Graph_Fini(gr);
    AF_Graph_free(gr);

    AF_Graphics_Finish(g);
    AF_Graphics_free(g);
  }

  /* done */
  return 0;
}



int AF_Report2_CreateCategoryReport(AE_BOOK *b, const AF_REPORT2_SETTINGS *settings, const char *fname)
{
  AF_GRAPH_DATASET *ds;

  /* prepare data set */
  ds=AF_Graph_DataSet_new();
  AF_Graph_DataSet_SetTitle(ds, AF_Report2_Settings_GetTitle(settings));
  AF_Graph_DataSet_SetSubTitle(ds, AF_Report2_Settings_GetSubTitle(settings));

  AF_Graph_DataSet_SetDataTypeX(ds, AF_Graph_DataSet_DataType_Double);
  AF_Graph_DataSet_SetPrecisionX(ds, 0);
  AF_Graph_DataSet_SetLabelX(ds, AF_Report2_Settings_GetLabelX(settings));

  AF_Graph_DataSet_SetDataTypeY(ds, AF_Graph_DataSet_DataType_Double);
  AF_Graph_DataSet_SetLabelY(ds, AF_Report2_Settings_GetLabelY(settings));
  AF_Graph_DataSet_SetPrecisionY(ds, AF_Report2_Settings_GetPrecisionY(settings));

  /* sample data into data set */
  {
    const AF_REPORT2_SETTINGS_GRAPH_LIST *sgList;
    const char *commodity;
    int rv;
    GWEN_DATE *firstDate=NULL;
    GWEN_DATE *lastDate=NULL;
    GWEN_DATE *today=GWEN_Date_CurrentDate();

    commodity=AF_Report2_Settings_GetCommodity(settings);
    if (!(commodity && *commodity))
      commodity="EUR";

    /* get dates */
    switch (AF_Report2_Settings_GetDateSpanType(settings)) {
    case AF_Report2_DateSpanType_None:
      break;

    case AF_Report2_DateSpanType_UserDefined: {
      const GWEN_DATE *dt;

      dt=AF_Report2_Settings_GetFirstDate(settings);
      if (dt)
        firstDate=GWEN_Date_dup(dt);
      dt=AF_Report2_Settings_GetLastDate(settings);
      if (dt)
        lastDate=GWEN_Date_dup(dt);
      break;
    }

    case AF_Report2_DateSpanType_ThisMonth:
      firstDate=GWEN_Date_GetThisMonthStart(today);
      lastDate=GWEN_Date_GetThisMonthEnd(today);
      break;

    case AF_Report2_DateSpanType_ThisQuarterYear:
      firstDate=GWEN_Date_GetThisQuarterYearStart(today);
      lastDate=GWEN_Date_GetThisQuarterYearEnd(today);
      break;

    case AF_Report2_DateSpanType_ThisHalfYear:
      firstDate=GWEN_Date_GetThisHalfYearStart(today);
      lastDate=GWEN_Date_GetThisHalfYearEnd(today);
      break;

    case AF_Report2_DateSpanType_ThisYear:
      firstDate=GWEN_Date_GetThisYearStart(today);
      lastDate=GWEN_Date_GetThisYearEnd(today);
      break;

    case AF_Report2_DateSpanType_LastMonth:
      firstDate=GWEN_Date_GetLastMonthStart(today);
      lastDate=GWEN_Date_GetLastMonthEnd(today);
      break;

    case AF_Report2_DateSpanType_LastQuarterYear:
      firstDate=GWEN_Date_GetLastQuarterYearStart(today);
      lastDate=GWEN_Date_GetLastQuarterYearEnd(today);
      break;

    case AF_Report2_DateSpanType_LastHalfYear:
      firstDate=GWEN_Date_GetLastHalfYearStart(today);
      lastDate=GWEN_Date_GetLastHalfYearEnd(today);
      break;

    case AF_Report2_DateSpanType_LastYear:
      firstDate=GWEN_Date_GetLastYearStart(today);
      lastDate=GWEN_Date_GetLastYearEnd(today);
      break;
    }

    DBG_ERROR(0, "Datespan is: %s - %s",
              firstDate?(GWEN_Date_GetString(firstDate)):"",
              lastDate?(GWEN_Date_GetString(lastDate)):"");

    sgList=AF_Report2_Settings_GetGraphList(settings);
    if (sgList) {
      const AF_REPORT2_SETTINGS_GRAPH *sg;
      int level=1;
      AE_ACCOUNT_TREE *at;

      at=AE_Account_Tree_new();
      rv=AE_Book_ReadAccountTree(b, AE_Book_TableType_Account, at);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        AE_Account_Tree_free(at);
        GWEN_Date_free(today);
        GWEN_Date_free(lastDate);
        GWEN_Date_free(firstDate);
        return rv;
      }

      sg=AF_Report2_SettingsGraph_List_First(sgList);
      while (sg) {
        if (AF_Report2_SettingsGraph_GetFlags(sg) & AF_REPORT2_FLAGS_ENABLED) {
          AF_REPORT2 *rp;
          AF_REPORT2_CATEGORY_TREE *catTree;
          AF_REPORT2_CATEGORY *cat;
          GWEN_STRINGLIST2 *sl;

          rp=AF_Report2_new();
          catTree=AF_Report2_GetCategoryTree(rp);
          rv=AF_Book_Report2_AccountTree2CategoryTree(b, catTree);
          if (rv<0) {
            DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
            AE_Account_Tree_free(at);
            GWEN_Date_free(today);
            GWEN_Date_free(lastDate);
            GWEN_Date_free(firstDate);
            return rv;
          }

          AE_Account_Tree_SubRuntimeFlags(at, AE_ACCOUNT_RTFLAGS_SELECTED);
          sl=AF_Report2_SettingsGraph_GetCategoryNameList(sg);
          if (sl) {
            GWEN_STRINGLIST2_ITERATOR *sli;
            AE_SPLIT_LIST *spList;

            /* set matching accounts SELECTED in account tree */
            sli=GWEN_StringList2_First(sl);
            if (sli) {
              const char *s;

              s=GWEN_StringList2Iterator_Data(sli);
              while (s) {
                AE_Account_Tree_SelectAccountAndChildrenByName(at, s, 1);
                s=GWEN_StringList2Iterator_Next(sli);
              }
              GWEN_StringList2Iterator_free(sli);
            }

            spList=AE_Split_List_new();
            rv=AE_Book_ReadSplitListForSelectedAccounts(b, at, firstDate, lastDate, commodity, spList);
            if (rv<0) {
              DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
              AE_Split_List_free(spList);
              AE_Account_Tree_free(at);
              AF_Report2_free(rp);
              GWEN_Date_free(today);
              GWEN_Date_free(lastDate);
              GWEN_Date_free(firstDate);
              return rv;
            }

            if (AE_Split_List_GetCount(spList)==0) {
              DBG_ERROR(AE_LOGDOMAIN, "No data for graph, aborting");
              AE_Split_List_free(spList);
              AE_Account_Tree_free(at);
              AF_Report2_free(rp);
              GWEN_Date_free(today);
              GWEN_Date_free(lastDate);
              GWEN_Date_free(firstDate);
              return GWEN_ERROR_NO_DATA;
            }


            AF_Book_Report2_SplitList2Report(b, rp, spList);
            AE_Split_List_free(spList);
          }

          /* sum up all categories */
          AF_Report2_Category_Tree_SumUpCategories(catTree);

          /* make selected category the new root for further handling */
          AF_Report2_Category_Tree_MarkCategoriesWithDataPairs(catTree);
          cat=AF_Report2_Category_Tree_FindHighestCommonRoot(catTree);
          if (cat) {
            DBG_ERROR(0, "Making element [%s] new root", AF_Report2_Category_GetName(cat));
            AF_Report2_Category_Tree_MakeElementNewRoot(catTree, cat);
          }
          else {
            DBG_ERROR(0, "No common root below tree root");
          }

          AF_Report2_AddCategoryData2DataSet(rp, ds,
                                             AF_Report2_SettingsGraph_GetFlags(sg),
                                             NULL,
                                             level,
                                             AF_Report2_SettingsGraph_GetGraphType(sg),
                                             AF_Report2_SettingsGraph_GetColour(sg),
                                             AF_Report2_SettingsGraph_GetLineWidth(sg),
                                             AF_Report2_SettingsGraph_GetDashType(sg));

          AF_Report2_free(rp);

          level++;
        }
        sg=AF_Report2_SettingsGraph_List_Next(sg);
      }

      AE_Account_Tree_free(at);
    }

    GWEN_Date_free(today);
    GWEN_Date_free(lastDate);
    GWEN_Date_free(firstDate);
  }

  /* prepare drawing of data set */
  AF_Graph_DataSet_AdjustXRange(ds);
  /* increment maxX and decrement maxY because otherwise the last tick label would not be drawn */
  AF_Graph_DataSet_SetMaxValueX(ds, AF_Graph_DataSet_GetMaxValueX(ds)+1);
  AF_Graph_DataSet_SetMinValueX(ds, AF_Graph_DataSet_GetMinValueX(ds)-1);
  AF_Graph_DataSet_AdjustYRange(ds);
  AF_Graph_DataSet_SetMaxValueY(ds, AF_Graph_DataSet_GetMaxValueY(ds)*1.10); /* add 10% */
  if (AF_Graph_DataSet_GetMinValueY(ds)>0.0)
    AF_Graph_DataSet_SetMinValueY(ds, 0.0);

  /* add ticks */
  {
    AF_GRAPH_TICK_LIST *tl;
    AF_REPORT2_DATAPAIR_LIST *dpl;
    int level=1;
    int dist;

    tl=AF_Graph_DataSet_GetYTickList(ds);
    if (tl==NULL) {
      tl=AF_Graph_Tick_List_new();
      AF_Graph_DataSet_SetYTickList(ds, tl);
    }
    AF_Graph_Tick_AddTicks(tl,
                           AF_Graph_DataSet_GetDataTypeY(ds),
                           AF_Graph_DataSet_GetMinValueY(ds),
                           AF_Graph_DataSet_GetMaxValueY(ds),
                           AF_Graph_DataSet_GetPrecisionY(ds));


    tl=AF_Graph_DataSet_GetXTickList(ds);
    if (tl==NULL) {
      tl=AF_Graph_Tick_List_new();
      AF_Graph_DataSet_SetXTickList(ds, tl);
    }

    /* add ticks for every data pair in the data set */
    dpl=AF_Graph_DataSet_GetDataPairList(ds);
    if (dpl) {
      AF_GRAPH_DATAPAIR *dp;

      dp=AF_Graph_DataPair_List_First(dpl);
      while (dp) {
        int idx;
        const char *label;
        AF_GRAPH_TICK *t;

        idx=AF_Graph_DataPair_GetValueX(dp);
        label=AF_Graph_DataPair_GetLabel(dp);

        t=AF_Graph_Tick_new();
        AF_Graph_Tick_SetValue(t, idx);
        AF_Graph_Tick_SetLevel(t, 1);
        AF_Graph_Tick_SetLabel(t, label);
        AF_Graph_Tick_List_Add(t, tl);

        dp=AF_Graph_DataPair_List_Next(dp);
      }
    }

  }


  /* actually draw */
  {
    AF_GRAPHICS *g;
    AF_GRAPH *gr;
    const AF_GRAPH_SETTINGS *gs;
    int rv;

    g=AF_GraphicsCairo_Png_new(fname, AF_Report2_Settings_GetWidth(settings), AF_Report2_Settings_GetHeight(settings));
    AF_Graphics_Attach(g);
    gs=AF_Report2_Settings_GetGraphSettings(settings);
    if (gs==NULL) {
      DBG_ERROR(AE_LOGDOMAIN, "No graph settings");
      AF_Graphics_free(g);
      return GWEN_ERROR_INVALID;
    }
    gr=AF_Graph_new(ds, g, AF_Report2_Settings_GetWidth(settings), AF_Report2_Settings_GetHeight(settings));
    rv=AF_Graph_Init(gr, gs);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AF_Graphics_free(g);
      AF_Graph_free(gr);
      return rv;
    }

    rv=AF_Graph_Draw(gr);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AF_Graphics_free(g);
      AF_Graph_free(gr);
      return rv;
    }

    AF_Graph_Fini(gr);
    AF_Graph_free(gr);

    AF_Graphics_Finish(g);
    AF_Graphics_free(g);
  }

  /* done */
  return 0;
}



int AF_Report2_CreateCounterReport(AE_BOOK *b, const AF_REPORT2_SETTINGS *settings, const char *fname)
{
  AF_GRAPH_DATASET *ds;

  /* prepare data set */
  ds=AF_Graph_DataSet_new();
  AF_Graph_DataSet_SetTitle(ds, AF_Report2_Settings_GetTitle(settings));
  AF_Graph_DataSet_SetSubTitle(ds, AF_Report2_Settings_GetSubTitle(settings));

  AF_Graph_DataSet_SetDataTypeX(ds, AF_Graph_DataSet_DataType_Date);
  AF_Graph_DataSet_SetLabelX(ds, I18N("Date"));

  AF_Graph_DataSet_SetDataTypeY(ds, AF_Graph_DataSet_DataType_Double);
  AF_Graph_DataSet_SetLabelY(ds, AF_Report2_Settings_GetLabelY(settings));
  AF_Graph_DataSet_SetPrecisionY(ds, AF_Report2_Settings_GetPrecisionY(settings));

  /* sample data into data set */
  {
    const AF_REPORT2_SETTINGS_GRAPH_LIST *sgList;
    const char *commodity;
    int rv;
    GWEN_DATE *firstDate=NULL;
    GWEN_DATE *lastDate=NULL;
    GWEN_DATE *today=GWEN_Date_CurrentDate();

    commodity=AF_Report2_Settings_GetCommodity(settings);
    if (!(commodity && *commodity))
      commodity="EUR";

    /* get dates */
    switch (AF_Report2_Settings_GetDateSpanType(settings)) {
    case AF_Report2_DateSpanType_None:
      break;

    case AF_Report2_DateSpanType_UserDefined: {
      const GWEN_DATE *dt;

      dt=AF_Report2_Settings_GetFirstDate(settings);
      if (dt)
        firstDate=GWEN_Date_dup(dt);
      dt=AF_Report2_Settings_GetLastDate(settings);
      if (dt)
        lastDate=GWEN_Date_dup(dt);
      break;
    }

    case AF_Report2_DateSpanType_ThisMonth:
      firstDate=GWEN_Date_GetThisMonthStart(today);
      lastDate=GWEN_Date_GetThisMonthEnd(today);
      break;

    case AF_Report2_DateSpanType_ThisQuarterYear:
      firstDate=GWEN_Date_GetThisQuarterYearStart(today);
      lastDate=GWEN_Date_GetThisQuarterYearEnd(today);
      break;

    case AF_Report2_DateSpanType_ThisHalfYear:
      firstDate=GWEN_Date_GetThisHalfYearStart(today);
      lastDate=GWEN_Date_GetThisHalfYearEnd(today);
      break;

    case AF_Report2_DateSpanType_ThisYear:
      firstDate=GWEN_Date_GetThisYearStart(today);
      lastDate=GWEN_Date_GetThisYearEnd(today);
      break;

    case AF_Report2_DateSpanType_LastMonth:
      firstDate=GWEN_Date_GetLastMonthStart(today);
      lastDate=GWEN_Date_GetLastMonthEnd(today);
      break;

    case AF_Report2_DateSpanType_LastQuarterYear:
      firstDate=GWEN_Date_GetLastQuarterYearStart(today);
      lastDate=GWEN_Date_GetLastQuarterYearEnd(today);
      break;

    case AF_Report2_DateSpanType_LastHalfYear:
      firstDate=GWEN_Date_GetLastHalfYearStart(today);
      lastDate=GWEN_Date_GetLastHalfYearEnd(today);
      break;

    case AF_Report2_DateSpanType_LastYear:
      firstDate=GWEN_Date_GetLastYearStart(today);
      lastDate=GWEN_Date_GetLastYearEnd(today);
      break;
    }


    sgList=AF_Report2_Settings_GetGraphList(settings);
    if (sgList) {
      const AF_REPORT2_SETTINGS_GRAPH *sg;
      int level=1;
      AE_COUNTER_TYPE_LIST *counterTypeList;

      /* read counter type list */
      counterTypeList=AE_CounterType_List_new();
      rv=AE_Book_ReadCounterTypeList(b, AE_Book_TableType_CounterType, counterTypeList);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        AE_CounterType_List_free(counterTypeList);
        GWEN_Date_free(today);
        GWEN_Date_free(lastDate);
        GWEN_Date_free(firstDate);
        return rv;
      }

      sg=AF_Report2_SettingsGraph_List_First(sgList);
      while (sg) {
        if (AF_Report2_SettingsGraph_GetFlags(sg) & AF_REPORT2_FLAGS_ENABLED) {
          AF_REPORT2 *rp;
          GWEN_STRINGLIST2 *sl;
          AE_COUNTER_TYPE *ct;

          rp=AF_Report2_new();

          /* add categories to report */
          ct=AE_CounterType_List_First(counterTypeList);
          while (ct) {
            AF_REPORT2_CATEGORY *cat;

            cat=AF_Report2_Category_new();
            AF_Report2_Category_SetId(cat, AE_CounterType_GetId(ct));
            AF_Report2_Category_SetName(cat, AE_CounterType_GetName(ct));
            rv=AF_Report2_AddCategory(rp, cat, 0);
            if (rv<0) {
              DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
              AE_CounterType_List_free(counterTypeList);
              GWEN_Date_free(today);
              GWEN_Date_free(lastDate);
              GWEN_Date_free(firstDate);
              return rv;
            }

            ct=AE_CounterType_List_Next(ct);
          }

          sl=AF_Report2_SettingsGraph_GetCategoryNameList(sg);
          if (sl) {
            GWEN_STRINGLIST2_ITERATOR *sli;

            sli=GWEN_StringList2_First(sl);
            if (sli) {
              const char *s;

              s=GWEN_StringList2Iterator_Data(sli);
              while (s) {
                AE_COUNTER_VALUE_LIST *valueList;
                AE_COUNTER_TYPE *t=NULL;
                AQDB_ID cid=0;
                char *copyOfS;
                char *p;
                const char *pCounterName=NULL;
                const char *pCounterFieldName=NULL;

                copyOfS=strdup(s);
                pCounterName=copyOfS;
                p=strchr(copyOfS, ':');
                if (p) {
                  *p=0;
                  pCounterFieldName=p+1;
                }

                if (*pCounterName=='#') {
                  long long unsigned int u;

                  /* get counter type by id */
                  if (1!=sscanf(pCounterName+1, "%llu", &u)) {
                    DBG_ERROR(AE_LOGDOMAIN, "Invalid Counter type id \"%s\"", pCounterName+1);
                    free(copyOfS);
                    GWEN_StringList2Iterator_free(sli);
                    AF_Report2_free(rp);
                    AE_CounterType_List_free(counterTypeList);
                    GWEN_Date_free(today);
                    GWEN_Date_free(lastDate);
                    GWEN_Date_free(firstDate);
                    return GWEN_ERROR_BAD_DATA;
                  }

                  /* get counter type by id */
                  t=AE_CounterType_List_GetById(counterTypeList, (AQDB_ID) u);
                  if (t==NULL) {
                    DBG_ERROR(AE_LOGDOMAIN, "Counter type \"%llu\" not found", u);
                    free(copyOfS);
                    GWEN_StringList2Iterator_free(sli);
                    AF_Report2_free(rp);
                    AE_CounterType_List_free(counterTypeList);
                    GWEN_Date_free(today);
                    GWEN_Date_free(lastDate);
                    GWEN_Date_free(firstDate);
                    return GWEN_ERROR_BAD_DATA;
                  }
                  cid=AE_CounterType_GetId(t);
                }
                else {
                  /* get counter type by name */
                  t=AE_CounterType_List_GetByName(counterTypeList, pCounterName);
                  if (t==NULL) {
                    DBG_ERROR(AE_LOGDOMAIN, "Counter type \"%s\" not found", pCounterName);
                    free(copyOfS);
                    GWEN_StringList2Iterator_free(sli);
                    AF_Report2_free(rp);
                    AE_CounterType_List_free(counterTypeList);
                    GWEN_Date_free(today);
                    GWEN_Date_free(lastDate);
                    GWEN_Date_free(firstDate);
                    return GWEN_ERROR_BAD_DATA;
                  }
                  cid=AE_CounterType_GetId(t);
                }

                /* read counter values */
                valueList=AE_CounterValue_List_new();
                rv=AE_Book_ReadCounterValueListByDateAndType(b, cid, NULL, NULL, valueList);
                if (rv<0) {
                  DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
                  AE_CounterValue_List_free(valueList);
                  free(copyOfS);
                  GWEN_StringList2Iterator_free(sli);
                  AF_Report2_free(rp);
                  AE_CounterType_List_free(counterTypeList);
                  GWEN_Date_free(today);
                  GWEN_Date_free(lastDate);
                  GWEN_Date_free(firstDate);
                  return rv;
                }
                else {
                  AE_COUNTER_VALUE *v;

                  /* extract counter fields and add to report */
                  AE_CounterValue_List_CalculateDiffValues(valueList);
                  AE_CounterValue_List_CalculateTotalValues(valueList);
                  v=AE_CounterValue_List_First(valueList);
                  while (v) {
                    const GWEN_DATE *dt;

                    dt=AE_CounterValue_GetDate(v);
                    if (dt) {
                      int doAdd=1;

                      if (doAdd && firstDate) {
                        if (GWEN_Date_GetJulian(dt)<GWEN_Date_GetJulian(firstDate))
                          doAdd=0;
                      }

                      if (doAdd && lastDate) {
                        if (GWEN_Date_GetJulian(dt)>GWEN_Date_GetJulian(lastDate))
                          doAdd=0;
                      }

                      if (doAdd) {
                        const AQDB_VALUE *amt;

                        amt=AE_CounterValues_GetFieldValueByName(v, pCounterFieldName);
                        if (amt) {
                          AF_REPORT2_DATAPAIR *dp;

                          dp=AF_Report2_DataPair_new();
                          AF_Report2_DataPair_SetCategoryId(dp, cid);
                          AF_Report2_DataPair_SetDate(dp, dt);
                          AF_Report2_DataPair_SetValue(dp, AQDB_Value_GetValueAsDouble(amt));

                          AF_Report2_AddDataPair(rp, dp);
                        }
                        else {
                          DBG_ERROR(AE_LOGDOMAIN, "Invalid field name \"%s\"", pCounterFieldName);
                          AE_CounterValue_List_free(valueList);
                          free(copyOfS);
                          GWEN_StringList2Iterator_free(sli);
                          AF_Report2_free(rp);
                          AE_CounterType_List_free(counterTypeList);
                          GWEN_Date_free(today);
                          GWEN_Date_free(lastDate);
                          GWEN_Date_free(firstDate);
                          return GWEN_ERROR_BAD_DATA;
                        }
                      }
                    }

                    v=AE_CounterValue_List_Next(v);
                  }
                }
                AE_CounterValue_List_free(valueList);
                free(copyOfS);

                s=GWEN_StringList2Iterator_Next(sli);
              }
              GWEN_StringList2Iterator_free(sli);
            }
          }

          AF_Report2_CollapseAllCategories(rp);

          switch (AF_Report2_Settings_GetDateUnits(settings)) {
          case AF_Report2_DateUnits_Days:
            AF_Report2_AddDayData2DataSet(rp, ds,
                                          AF_Report2_SettingsGraph_GetFlags(sg),
                                          NULL,
                                          level,
                                          AF_Report2_SettingsGraph_GetGraphType(sg),
                                          AF_Report2_SettingsGraph_GetDataMode(sg),
                                          AF_Report2_SettingsGraph_GetColour(sg),
                                          AF_Report2_SettingsGraph_GetLineWidth(sg),
                                          AF_Report2_SettingsGraph_GetDashType(sg));
            break;

          default:
          case AF_Report2_DateUnits_Months:
            AF_Report2_AddMonthData2DataSet(rp, ds,
                                            AF_Report2_SettingsGraph_GetFlags(sg),
                                            NULL,
                                            level,
                                            AF_Report2_SettingsGraph_GetGraphType(sg),
                                            AF_Report2_SettingsGraph_GetDataMode(sg),
                                            AF_Report2_SettingsGraph_GetColour(sg),
                                            AF_Report2_SettingsGraph_GetLineWidth(sg),
                                            AF_Report2_SettingsGraph_GetDashType(sg));
            break;

#if 0
          case AF_Report2_DateUnits_Years:
            AF_Report2_AddYearData2DataSet(rp, ds,
                                           AF_Report2_SettingsGraph_GetFlags(sg),
                                           NULL,
                                           level,
                                           AF_Report2_SettingsGraph_GetGraphType(sg),
                                           AF_Report2_SettingsGraph_GetDataMode(sg),
                                           AF_Report2_SettingsGraph_GetColour(sg),
                                           AF_Report2_SettingsGraph_GetLineWidth(sg),
                                           AF_Report2_SettingsGraph_GetDashType(sg));
            break;
#endif
          }
          AF_Report2_free(rp);

          level++;
        }
        sg=AF_Report2_SettingsGraph_List_Next(sg);
      }

      AE_CounterType_List_free(counterTypeList);
    }

    GWEN_Date_free(today);
    GWEN_Date_free(lastDate);
    GWEN_Date_free(firstDate);
  }

  /* prepare drawing of data set */
  AF_Graph_DataSet_AdjustXRange(ds);
  AF_Graph_DataSet_AdjustYRange(ds);


  /* add ticks */
#if 0
  AF_Graph_DataSet_AddTicks(ds);
#else
  {
    AF_GRAPH_TICK_LIST *tl;
    int level=1;
    int dist;

    tl=AF_Graph_DataSet_GetYTickList(ds);
    if (tl==NULL) {
      tl=AF_Graph_Tick_List_new();
      AF_Graph_DataSet_SetYTickList(ds, tl);
    }
    AF_Graph_Tick_AddTicks(tl,
                           AF_Graph_DataSet_GetDataTypeY(ds),
                           AF_Graph_DataSet_GetMinValueY(ds),
                           AF_Graph_DataSet_GetMaxValueY(ds),
                           AF_Graph_DataSet_GetPrecisionY(ds));


    tl=AF_Graph_DataSet_GetXTickList(ds);
    if (tl==NULL) {
      tl=AF_Graph_Tick_List_new();
      AF_Graph_DataSet_SetXTickList(ds, tl);
    }

    dist=AF_Graph_DataSet_GetMaxValueX(ds)-AF_Graph_DataSet_GetMinValueX(ds);

    /* add X ticks */
    switch (AF_Report2_Settings_GetDateUnits(settings)) {
    case AF_Report2_DateUnits_Days:
      if (dist>365)
        AF_Graph_Tick_AddTicks_DateYears(tl, level++, AF_Graph_DataSet_GetMinValueX(ds), AF_Graph_DataSet_GetMaxValueX(ds));
      if (dist>30)
        AF_Graph_Tick_AddTicks_DateMonths(tl, level++, AF_Graph_DataSet_GetMinValueX(ds), AF_Graph_DataSet_GetMaxValueX(ds));
      if (dist>7)
        AF_Graph_Tick_AddTicks_DateWeeks(tl, level++, AF_Graph_DataSet_GetMinValueX(ds), AF_Graph_DataSet_GetMaxValueX(ds));
      if (dist>1)
        AF_Graph_Tick_AddTicks_DateDays(tl, level++, AF_Graph_DataSet_GetMinValueX(ds), AF_Graph_DataSet_GetMaxValueX(ds));
      break;

    default:
    case AF_Report2_DateUnits_Months:
      if (dist>365)
        AF_Graph_Tick_AddTicks_DateYears(tl, level++, AF_Graph_DataSet_GetMinValueX(ds), AF_Graph_DataSet_GetMaxValueX(ds));
      if (dist>30)
        AF_Graph_Tick_AddTicks_DateMonths(tl, level++, AF_Graph_DataSet_GetMinValueX(ds), AF_Graph_DataSet_GetMaxValueX(ds));
      break;

    case AF_Report2_DateUnits_Years:
      if (dist>365)
        AF_Graph_Tick_AddTicks_DateYears(tl, level++, AF_Graph_DataSet_GetMinValueX(ds), AF_Graph_DataSet_GetMaxValueX(ds));
      break;
    }
  }
#endif


  /* actually draw */
  {
    AF_GRAPHICS *g;
    AF_GRAPH *gr;
    const AF_GRAPH_SETTINGS *gs;
    int rv;

    g=AF_GraphicsCairo_Png_new(fname, AF_Report2_Settings_GetWidth(settings), AF_Report2_Settings_GetHeight(settings));
    AF_Graphics_Attach(g);
    gs=AF_Report2_Settings_GetGraphSettings(settings);
    if (gs==NULL) {
      DBG_ERROR(AE_LOGDOMAIN, "No graph settings");
      AF_Graphics_free(g);
      return GWEN_ERROR_INVALID;
    }
    gr=AF_Graph_new(ds, g, AF_Report2_Settings_GetWidth(settings), AF_Report2_Settings_GetHeight(settings));
    rv=AF_Graph_Init(gr, gs);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AF_Graphics_free(g);
      AF_Graph_free(gr);
      return rv;
    }

    rv=AF_Graph_Draw(gr);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AF_Graphics_free(g);
      AF_Graph_free(gr);
      return rv;
    }

    AF_Graph_Fini(gr);
    AF_Graph_free(gr);

    AF_Graphics_Finish(g);
    AF_Graphics_free(g);
  }

  /* done */
  return 0;
}










